@@ -9,11 +9,15 @@ NO_OF_THREADS = 8 # default no of threads is 8
 SUPPORTED_BACKENDS = ['llvm', 'llvm2', 'llvm_rtlib', 'c', 'cpp', 'x86', 'wasm',
                       'gfortran', 'llvmImplicit', 'llvmStackArray', 'fortran',
                       'c_nopragma', 'llvm_nopragma', 'llvm_wasm', 'llvm_wasm_emcc',
-                      'llvm_omp', 'mlir']
+                      'llvm_omp', 'llvm_submodule', 'mlir', 'mlir_omp', 'mlir_llvm_omp',
+                      'llvm_goc', 'target_offload', 'llvm_single_invocation']
+SUPPORTED_STANDARDS = ['lf', 'f23', 'legacy']
 BASE_DIR = os.path.dirname(os.path.realpath(__file__))
 LFORTRAN_PATH = f"{BASE_DIR}/../src/bin:$PATH"
 
 fast_tests = "no"
+nofast_llvm16 = "no"
+separate_compilation = "no"
 
 def run_cmd(cmd, cwd=None):
     print(f"+ {cmd}")
@@ -22,32 +26,46 @@ def run_cmd(cmd, cwd=None):
         print("Command failed.")
         exit(1)
 
-def run_test(backend):
+def run_test(backend, std):
     run_cmd(f"mkdir {BASE_DIR}/test-{backend}")
+    if std == "f23":
+        std_string = "-DSTD_F23=yes"
+    elif std == "legacy":
+        std_string = "-DSTD_LEGACY=yes"
+    elif std == "lf":
+        std_string = ""
+    else:
+        raise Exception("Unsupported standard")
+
     cwd=f"{BASE_DIR}/test-{backend}"
     common=f" -DCURRENT_BINARY_DIR={BASE_DIR}/test-{backend} -S {BASE_DIR} -B {BASE_DIR}/test-{backend}"
     if backend == "gfortran":
         run_cmd(f"FC=gfortran cmake" + common,
                 cwd=cwd)
     elif backend == "cpp":
-        run_cmd(f"FC=lfortran FFLAGS=\"--openmp\" cmake -DLFORTRAN_BACKEND={backend} -DFAST={fast_tests} -DEXPERIMENTAL_SIMPLIFIER={experimental_simplifier}" + common,
+        run_cmd(f"FC=lfortran FFLAGS=\"--openmp\" cmake -DLFORTRAN_BACKEND={backend} -DFAST={fast_tests} "
+                f"-DLLVM_GOC={separate_compilation} -DNOFAST_LLVM16={nofast_llvm16} {std_string}" + common,
                 cwd=cwd)
     elif backend == "fortran":
         run_cmd(f"FC=lfortran cmake -DLFORTRAN_BACKEND={backend} "
-            f"-DFAST={fast_tests} -DCMAKE_Fortran_FLAGS=\"-fPIC\" -DEXPERIMENTAL_SIMPLIFIER={experimental_simplifier}" + common,
+            f"-DFAST={fast_tests} -DLLVM_GOC={separate_compilation} -DNOFAST_LLVM16={nofast_llvm16} "
+            f"-DCMAKE_Fortran_FLAGS=\"-fPIC\" {std_string}" + common,
                 cwd=cwd)
     else:
-        run_cmd(f"FC=lfortran cmake -DLFORTRAN_BACKEND={backend} -DFAST={fast_tests} -DEXPERIMENTAL_SIMPLIFIER={experimental_simplifier}" + common,
+        run_cmd(f"FC=lfortran cmake -DLFORTRAN_BACKEND={backend} -DFAST={fast_tests} "
+                f"-DLLVM_GOC={separate_compilation} {std_string} -DNOFAST_LLVM16={nofast_llvm16} " + common,
                 cwd=cwd)
     run_cmd(f"make -j{NO_OF_THREADS}", cwd=cwd)
     run_cmd(f"ctest -j{NO_OF_THREADS} --output-on-failure", cwd=cwd)
 
 
-def test_backend(backend):
+def test_backend(backend, std):
     if backend not in SUPPORTED_BACKENDS:
-        print(f"Unsupported Backend: {backend}\n")
-        return
-    run_test(backend)
+        raise Exception(f"Unsupported Backend: {backend}\n")
+    if std not in SUPPORTED_STANDARDS:
+        raise Exception(f"Unsupported Backend: {std}\n")
+
+    run_test(backend, std)
 
 def check_module_names():
     from glob import glob
@@ -78,12 +96,16 @@ def get_args():
     parser.add_argument("-b", "--backends", nargs="*", default=["llvm"], type=str,
                 help="Test the requested backends (%s)" % \
                         ", ".join(SUPPORTED_BACKENDS))
+    parser.add_argument("--std", type=str, default="lf",
+                help="Run tests with the requested Fortran standard: ".join(SUPPORTED_STANDARDS))
     parser.add_argument("-f", "--fast", action='store_true',
                 help="Run supported tests with --fast")
+    parser.add_argument("-sc", "--separate_compilation", action='store_true',
+                help="Run tests with --separate-compilation")
+    parser.add_argument("-nf16", "--no_fast_till_llvm16", action='store_true',
+                help="Don't run unsupported tests with --fast when LLVM < 17")
     parser.add_argument("-m", action='store_true',
                 help="Check that all module names are unique")
-    parser.add_argument("--experimental-simplifier",
-                        action='store_true', help="Use simplifier ASR pass")
     return parser.parse_args()
 
 def main():
@@ -94,17 +116,20 @@ def main():
         return
 
     # Setup
-    global NO_OF_THREADS, fast_tests, experimental_simplifier
+    global NO_OF_THREADS, fast_tests, std_f23_tests, nofast_llvm16, separate_compilation
     os.environ["PATH"] += os.pathsep + LFORTRAN_PATH
+    # Set environment variable for testing
+    os.environ["LFORTRAN_TEST_ENV_VAR"] = "STATUS OK!"
     # delete previously created directories (if any)
     for backend in SUPPORTED_BACKENDS:
         run_cmd(f"rm -rf {BASE_DIR}/test-{backend}")
 
     NO_OF_THREADS = args.no_of_threads or NO_OF_THREADS
     fast_tests = "yes" if args.fast else "no"
-    experimental_simplifier = "yes" if args.experimental_simplifier else "no"
+    nofast_llvm16 = "yes" if args.no_fast_till_llvm16 else "no"
+    separate_compilation = "yes" if args.separate_compilation else "no"
     for backend in args.backends:
-        test_backend(backend)
+        test_backend(backend, args.std)
 
 if __name__ == "__main__":
     main()
