@@ -4,9 +4,10 @@ all whitespace.  It uses a hand written recursive descent parser to figure out
 how to properly tokenize the input. It returns a list of tokens that are then
 fed into our Bison parser, that is shared with the free-form parser.
 
-Note: The prescanner removes CR, so we only handle LF here.
+Note: The prescanner removes CR from regular code, but not from string literals.
 */
-#include <limits>
+#include <algorithm>
+#include <unordered_map>
 #include <utility>
 
 #include <lfortran/parser/parser_exception.h>
@@ -14,6 +15,7 @@ Note: The prescanner removes CR, so we only handle LF here.
 #include <lfortran/parser/parser.tab.hh>
 #include <lfortran/parser/tokenizer.h>
 #include <libasr/bigint.h>
+#include <libasr/string_utils.h>
 
 #include <lfortran/pickle.h>
 
@@ -23,7 +25,7 @@ int position = 0;
 
 namespace LCompilers::LFortran {
 
-std::map<std::string, yytokentype> identifiers_map = {
+const std::unordered_map<std::string, yytokentype> identifiers_map = {
     {"EOF", END_OF_FILE},
     {"\n", TK_NEWLINE},
     {"name", TK_NAME},
@@ -267,15 +269,21 @@ std::map<std::string, yytokentype> identifiers_map = {
     {"uminus", UMINUS}
 };
 
-std::vector<std::string> declarators{
+// star-forms must appear before non-stars
+const std::vector<std::string> declarators{
+            "integer*",
             "integer",
+	    "real*",
             "real",
+	    "complex*",
             "complex",
             "doubleprecision",
             "doublecomplex",
             "external",
             "dimension",
+	    "character*",
             "character",
+	    "logical*",
             "logical",
             "bytes",
             "data",
@@ -328,29 +336,30 @@ struct FixedFormRecursiveDescent {
         loc.last = loc_first;
         // auto next=cur; next_line(next);
         //std::cout << "error line " << tostr(cur,next-1) << std::endl;
-        throw LFortran::parser_local::TokenizerError(text, loc);
+        diag.add(diag::Diagnostic(
+            text,
+            diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+        throw parser_local::TokenizerAbort();
     }
 
     // Are the next characters in the `cur` stream equal to `str`?
-    bool next_is(unsigned char *cur, const std::string &str) {
-        unsigned char *tok = cur;
-        unsigned char *cur2 = cur;
-        while ((size_t)(cur2-tok) < str.size()) {
-            if (*cur2 == '\0') {
-                return false;
-            }
-            cur2++;
-        }
-        std::string next_str = std::string((char *)tok, cur2 - tok);
-        return next_str == str;
+    bool next_is(unsigned char const *cur, const std::string &str) {
+	for(const char s : str) {
+	    if (!s || *cur++ != s) return false;
+	}
+	return true;
     }
 
-    bool next_is_eol(unsigned char *cur) {
-        if (*cur == '\n') {
-            return true;
-        } else {
-            return false;
-        }
+    // Are the next characters in the `cur` stream equal to `str`?
+    constexpr bool next_is(unsigned char const * cur, const char *str) {
+	while(*str) {
+	    if (*cur++ != *str++) return false;
+	}
+	return true;
+    }
+
+    constexpr bool next_is_eol(unsigned char const *cur) {
+        return (*cur == '\n');
     }
 
     bool is_integer(const std::string &s) const {
@@ -426,6 +435,10 @@ struct FixedFormRecursiveDescent {
 
     void next_line(unsigned char *&cur) {
         while (*cur != '\n' && *cur != '\0') {
+            if (*cur == '\r') {
+                cur++;
+                continue;
+            }
             cur++;
         }
         if (*cur == '\n') cur++;
@@ -434,7 +447,7 @@ struct FixedFormRecursiveDescent {
     // Push the token_type, YYSTYPE and Location of the token_str at `cur`.
     // (Does not modify `cur`.)
     void push_token_no_advance_token(unsigned char *cur, const std::string &token_str,
-            yytokentype token_type) {
+            yytokentype const token_type) {
         YYSTYPE yy;
         yy.string.from_str(m_a, token_str);
         stypes.push_back(yy);
@@ -447,7 +460,9 @@ struct FixedFormRecursiveDescent {
 
     // token_type automatically determined
     void push_token_no_advance(unsigned char *cur, const std::string &token_str) {
-        push_token_no_advance_token(cur, token_str, identifiers_map[token_str]);
+	auto it = identifiers_map.find(token_str);
+	LCOMPILERS_ASSERT(it != identifiers_map.end());
+        push_token_no_advance_token(cur, token_str, it->second);
     }
 
     void push_integer_no_advance(unsigned char *cur, int32_t n) {
@@ -614,6 +629,10 @@ struct FixedFormRecursiveDescent {
                 // String is ended by delim
                 return true;
             }
+            if (*cur == '\r') {
+                cur++;
+                continue;
+            }
             cur++;
         }
     }
@@ -722,7 +741,7 @@ struct FixedFormRecursiveDescent {
         ptrdiff_t len;
         while (t.cur < end) {
             YYSTYPE y2;
-            auto token = t.lex(m_a, y2, loc, diag);
+            auto token = t.lex(m_a, y2, loc, diag, false);
             len = t.cur - t.tok;
             tokens.push_back(token);
             if (token == yytokentype::TK_INTEGER) {
@@ -731,7 +750,13 @@ struct FixedFormRecursiveDescent {
                     y2.int_suffix.int_kind);
             } else if (token == yytokentype::TK_STRING) {
                 std::string tk{tostr(t.tok+1, t.tok + len-1)};
-                y2.string.from_str(m_a, tk);
+                tk.erase(std::remove(tk.begin(), tk.end(), '\r'), tk.end());
+                Str s;
+                s.from_str(m_a, tk);
+                char quote_char = t.tok[0];
+                char* unescaped = str_unescape_fortran(m_a, s, quote_char);
+                y2.string.p = unescaped;
+                y2.string.n = strlen(unescaped);
             } else {
                 std::string tk{tostr(t.tok, t.tok + len)};
                 y2.string.from_str(m_a, tk);
@@ -762,13 +787,19 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = end - string_start;
                 loc.last = end - string_start;
-                throw parser_local::TokenizerError("Expected `)` here to end the condition expression of the if statement ", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected `)` here to end the condition expression of the if statement ",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             }
         } else {
             Location loc;
             loc.first = cur - string_start;
             loc.last = cur - string_start;
-            throw parser_local::TokenizerError("Expected expression after `if`", loc);
+            diag.add(diag::Diagnostic(
+                "Expected expression after `if`",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
         tokenize_until(end);
         cur = end;
@@ -790,9 +821,49 @@ struct FixedFormRecursiveDescent {
         return multiline;
     }
 
+    // Determines if a line in Fortran code represents an assignment rather than a declaration.
+    // E.g. below are declarations
+    //      integer :: x = 1
+    //      DATA p(1), (c(i), i=1, 2) / 5, 10, 12 /
+    //
+    // while, below are assignments
+    //      i = 2
+    //      integerx = 12
+    //      datap = 5
+    // NOTE: this function doesn't actually lexes, but only determines if it's
+    // (possibly) an assignment or not
+    bool is_possible_assignment(unsigned char *start, unsigned char *&cur) {
+        unsigned char *end = start;
+        if (!try_name(end)) return false;
+
+        // Try parsing array indices, if any
+        while (*end == '(') {
+            end++;  // Move past '('
+            if (!try_expr(end, true)) {
+                return false;  // Parsing failed, it’s not an assignment
+            }
+            if (*end != ')') {
+                return false;  // Expected closing ')', not an assignment
+            }
+            end++;  // Move past ')'
+        }
+
+        // After parsing identifier and indices, check if the next character is `=`
+        if (*end == '=') {
+            cur = start;  // Reset to the start to re-tokenize as an assignment
+            return true;
+        }
+        return false;  // Not an assignment if no '=' found
+    }
+
     bool lex_declaration(unsigned char *&cur) {
         unsigned char *start = cur;
         next_line(cur);
+
+        if (is_possible_assignment(start, cur)) {
+            return false;
+        }
+
         if (lex_declarator(start)) {
             tokenize_line(start);
             return true;
@@ -804,7 +875,20 @@ struct FixedFormRecursiveDescent {
     bool lex_declarator(unsigned char *&cur) {
         for(const auto& declarator : declarators) {
             if(next_is(cur, declarator)) {
-                push_token_advance(cur, declarator);
+		if (declarator.back() == '*') {
+		    /* In "star" form declarations, we need to be careful of only parsing an integer (no kind)
+		       after the star, so that we correctly parse "character*2d3v" as "character*2 d3v"
+		       instead of "character*2d3 v", which is illegal. */
+		    push_token_advance(cur, declarator.substr(0, declarator.size() - 1));
+		    push_token_advance(cur, std::string("*"));
+		    unsigned char *int_start = cur;
+		    if (try_integer(cur)) {
+			int32_t val = std::atoi((char*)int_start);
+			push_integer_no_advance(int_start, val);
+		    }
+		} else {
+		    push_token_advance(cur, declarator);
+		}
                 return true;
             }
         }
@@ -818,7 +902,7 @@ struct FixedFormRecursiveDescent {
                     cur += io_str.size();
                     unsigned char *start;
                     Location loc;
-                    lex_format(cur, loc, start);
+                    lex_format(cur, loc, start, diag, false, this->string_start);
                     locations.push_back(loc);
                     YYSTYPE yylval;
                     yylval.string.p = (char*) start;
@@ -903,7 +987,7 @@ struct FixedFormRecursiveDescent {
             if (*(cur+1) == '/' && !first_slash) {
                 if (*cur != ',') {
                     cur+=1;
-                } 
+                }
                 unsigned char *end = cur;
                 // tokenize uptil here
                 tokenize_until(end);
@@ -940,11 +1024,33 @@ struct FixedFormRecursiveDescent {
         return true;
     }
 
-    bool lex_body_statement(unsigned char *&cur) {
+    void lex_derived_type(unsigned char *&cur) {
+        push_token_advance(cur, "type");
+        tokenize_line(cur);
+        while (true) {
+            if (next_is(cur, "endtype")) {
+                push_token_advance(cur, "endtype");
+                tokenize_line(cur);
+                break;
+            } else {
+                lex_declaration(cur);
+            }
+        }
+    }
+
+    bool lex_body_statement(unsigned char *&cur, bool continue_compilation = false) {
         int64_t l = eat_label(cur);
+        // handle derived type tokenization
+        // this needs to be done before 'lex_declaration'
+        if (next_is(cur, "type::")) {
+            lex_derived_type(cur);
+            return true;
+        }
+
         if (lex_declaration(cur)) {
             return true;
         }
+
         if (lex_io(cur)) return true;
         if (next_is(cur, "if(")) {
             lex_cond(cur);
@@ -956,6 +1062,11 @@ struct FixedFormRecursiveDescent {
             return true;
         }
 
+        if (next_is(cur, "doconcurrent(")) {
+            lex_do_concurrent(cur);
+            return true;
+        }
+
         if (next_is(cur, "dowhile(")) {
             lex_dowhile(cur);
             return true;
@@ -994,12 +1105,53 @@ struct FixedFormRecursiveDescent {
             return true;
         }
 
+        if (next_is(cur, "backspace")) {
+            push_token_advance(cur, "backspace");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "rewind")) {
+            push_token_advance(cur, "rewind");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "endfile")) {
+            push_token_advance(cur, "endfile");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "interface")) {
+            lex_interface(cur);
+            return true;
+        }
+
         if (next_is(cur, "exit")) {
             push_token_advance(cur, "exit");
             tokenize_line(cur);
             return true;
         }
 
+        if (next_is(cur, "flush")) {
+            push_token_advance(cur, "flush");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "allocate")) {
+            push_token_advance(cur, "allocate");
+            tokenize_line(cur);
+            return true;
+        }
+
+        if (next_is(cur, "deallocate")) {
+            push_token_advance(cur, "deallocate");
+            tokenize_line(cur);
+            return true;
+        }
+
         if (next_is(cur, "goto")) {
             push_token_advance(cur, "goto");
             tokenize_line(cur);
@@ -1077,6 +1229,13 @@ struct FixedFormRecursiveDescent {
             // Undo the label, as it will be handled later
             undo_label(cur);
         }
+        if (next_is(cur, "endprogram") || next_is(cur, "end") || next_is(cur, "contains") || next_is(cur, "subroutine") || next_is(cur, "function")) {
+            return false;
+        }
+        if (continue_compilation) {
+            tokenize_line(cur);
+            return true;
+        }
 
         return false;
     }
@@ -1092,13 +1251,19 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected 'to' here", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected 'to' here",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             }
         } else {
             Location loc;
             loc.first = cur-string_start;
             loc.last = cur-string_start;
-            throw parser_local::TokenizerError("Expected integer after `assign`", loc);
+            diag.add(diag::Diagnostic(
+                "Expected integer after `assign`",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
     }
 
@@ -1127,7 +1292,10 @@ struct FixedFormRecursiveDescent {
             Location loc;
             loc.first = 1;
             loc.last = 1;
-            throw parser_local::TokenizerError("End of file encountered in labeled do loop (loop is not terminated)", loc);
+            diag.add(diag::Diagnostic(
+                "End of file encountered in labeled do loop (loop is not terminated)",
+                diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+            throw parser_local::TokenizerAbort();
         }
         int64_t label = eat_label(cur);
         if (label != do_label) {
@@ -1250,6 +1418,29 @@ struct FixedFormRecursiveDescent {
         return false;
     }
 
+    void lex_do_concurrent(unsigned char *&cur) {
+        push_token_advance(cur, "do");
+        push_token_advance(cur, "concurrent");
+        tokenize_line(cur);
+
+        while (true) {
+            if (next_is(cur, "enddo")) {
+                push_token_no_advance(cur, "enddo");
+                push_token_no_advance(cur, "\n");
+                next_line(cur);
+                break;
+            } else if (!lex_body_statement(cur)) {
+                Location loc;
+                loc.first = cur - string_start;
+                loc.last = cur - string_start;
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside do concurrent loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
+            }
+        }
+    }
+
     void lex_do(unsigned char *&cur) {
         auto end = cur; next_line(end);
         push_token_advance(cur, "do");
@@ -1293,7 +1484,10 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected an executable statement inside a do loop", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside a do loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             };
         }
     }
@@ -1308,7 +1502,10 @@ struct FixedFormRecursiveDescent {
                 Location loc;
                 loc.first = cur-string_start;
                 loc.last = cur-string_start;
-                throw parser_local::TokenizerError("Expected an executable statement inside a labeled do loop", loc);
+                diag.add(diag::Diagnostic(
+                    "Expected an executable statement inside a labeled do loop",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
             };
             if (!label_last(do_label)) {
                 // The lex_body_statement() above was a labeled loop that ended
@@ -1385,6 +1582,12 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "continue");
             push_token_no_advance(cur, "\n");
         }
+        if (next_is(cur, "contains")) {
+            push_token_advance(cur, "contains");
+            push_token_no_advance(cur, "\n");
+            next_line(cur);
+            while(lex_procedure(cur));
+        }
         if (next_is(cur, "endsubroutine")) {
             push_token_advance(cur, "endsubroutine");
             tokenize_line(cur);
@@ -1396,18 +1599,55 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    void lex_program(unsigned char *&cur, bool explicit_program) {
+    /*
+        explicit_program: This flag indicates whether the Fortran
+            program being lexed is explicitly declared with the
+            program keyword or implicitly defined by the presence
+            of statements that belong to a main program but without
+            an explicit program declaration.
+
+            E.g.; program where `explicit_program` is false:
+            ```
+                   integer :: x
+                   x = 42
+                   print *, x
+                   end
+            ```
+    */
+    void lex_program(unsigned char *&cur, bool explicit_program, bool continue_compilation, diag::Diagnostics &diagnostics) {
         if (explicit_program) {
             push_token_advance(cur, "program");
             tokenize_line(cur);
         }
-        while(lex_body_statement(cur));
+        while(lex_body_statement(cur, continue_compilation));
         eat_label(cur);
         if (next_is(cur, "contains")) {
             push_token_advance(cur, "contains");
             push_token_no_advance(cur, "\n");
             next_line(cur); // Does not generate any code?
             while(lex_procedure(cur));
+        } else if (next_is(cur, "subroutine") || next_is(cur, "function")) {
+            Location loc;
+            token_loc(cur, cur + 1, loc);
+            if (continue_compilation) {
+                diagnostics.add(diag::Diagnostic(
+                    "Expecting contains keyword before procedure definition",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                if (next_is(cur, "subroutine")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                } else if (next_is(cur, "function")) {
+                    while (!next_is(cur, "endprogram")) {
+                        next_line(cur);
+                    }
+                }
+            } else {
+                diagnostics.add(diag::Diagnostic(
+                    "Expecting contains keyword before procedure definition",
+                    diag::Level::Error, diag::Stage::Tokenizer, {diag::Label("", {loc})}));
+                throw parser_local::TokenizerAbort();
+            }
         }
         if (next_is(cur, "endprogram")) {
             push_token_advance(cur, "endprogram");
@@ -1420,6 +1660,28 @@ struct FixedFormRecursiveDescent {
         }
     }
 
+    void lex_module(unsigned char *&cur) {
+        push_token_advance(cur, "module");
+        tokenize_line(cur);
+        while(lex_body_statement(cur));
+        eat_label(cur);
+        if (next_is(cur, "contains")) {
+            push_token_advance(cur, "contains");
+            push_token_no_advance(cur, "\n");
+            next_line(cur);
+            while(lex_procedure(cur));
+        }
+        if (next_is(cur, "endmodule")) {
+            push_token_advance(cur, "endmodule");
+            tokenize_line(cur);
+        } else if (next_is(cur, "end")) {
+            push_token_advance(cur, "end");
+            tokenize_line(cur);
+        } else {
+            error(cur, "Expecting terminating symbol for module");
+        }
+    }
+
     void lex_function(unsigned char *&cur) {
         while(lex_body_statement(cur));
         int64_t l = eat_label(cur);
@@ -1438,6 +1700,26 @@ struct FixedFormRecursiveDescent {
         }
     }
 
+    void lex_interface(unsigned char *&cur) {
+        push_token_advance(cur, "interface");
+        tokenize_line(cur);
+
+        while (true) {
+            if (next_is(cur, "endinterface")) {
+                push_token_advance(cur, "endinterface");
+                tokenize_line(cur);
+                break;
+            } else if (next_is(cur, "subroutine") || next_is(cur, "function")) {
+                // Handle procedure declaration within the interface
+                lex_procedure(cur);
+            } else if (next_is(cur, "moduleprocedure")) {
+                // TODO: handle module procedure
+            } else {
+                error(cur, "Unexpected token in interface block");
+            }
+        }
+    }
+
     void lex_block_data(unsigned char *&cur) {
         push_token_advance(cur, "block");
         push_token_advance(cur, "data");
@@ -1454,12 +1736,14 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    bool is_declaration(unsigned char *&cur, std::string declaration_type /*function, subroutine, program*/, const std::vector<std::string>& keywords) {
+    bool is_declaration(unsigned char *&cur,
+			std::string declaration_type /*function, subroutine, program*/,
+			const std::vector<std::string>& keywords) {
         unsigned char *cpy = cur;
         unsigned char *nextline = cur; next_line(nextline);
         std::string line{tostr(cur, nextline-1)};
         // current line does not contain type -> we abort
-        if (!(line.find(std::string(declaration_type)) != std::string::npos)) return false;
+        if (!(line.find(declaration_type) != std::string::npos)) return false;
         std::vector<std::string> kw_found;
         std::vector<std::string> decls{keywords.begin(), keywords.end()};
         while(decls.size() != 0) {
@@ -1467,16 +1751,32 @@ struct FixedFormRecursiveDescent {
                 if (next_is(cpy, decls[i])) {
                     kw_found.push_back(decls[i]);
                     cpy += decls[i].size();
+                    if (decls[i].back() == '*') {
+ 		        if (decls[i] == "character*" && next_is(cpy, "(*)")) {
+			    kw_found.back() += "(*)";
+			    cpy += 3;
+                        } else if (std::isdigit(*cpy)) {
+			    do {
+                                kw_found.back().push_back(*cpy++);
+                            } while(std::isdigit(*cpy));
+			} else {
+			    error(cpy, "Syntax error: expecting length "
+				  "specification after " + decls[i]);
+			}
+                    }
                     decls.erase(decls.begin() + i);
                     break;
                 }
             }
             if (next_is(cpy, declaration_type))
                 break;
-            // catching syntax errors like `recursive double precision recursive function f(...`
+            // catching syntax errors like `recursive double precision
+            // recursive function f(...`
             for (auto kw = kw_found.begin(); kw != kw_found.end(); ++kw) {
                 if (next_is(cpy, *kw)) {
-                    error(cpy, "Syntax error: keyword " + *kw + "cannot occur multiple times in " + declaration_type + "declaration");
+                    error(cpy, "Syntax error: keyword " + *kw +
+			  "cannot occur multiple times in " +
+			  declaration_type + "declaration");
                 }
             }
         }
@@ -1484,11 +1784,11 @@ struct FixedFormRecursiveDescent {
             return false;
 
         // tokenize all keywords
-        for(auto iter = kw_found.begin(); iter != kw_found.end(); ++iter) {
-            if (*iter == "real*8" || *iter == "complex*8" || *iter == "complex*16" || *iter == "character*1") {
-                tokenize_until(cur+(*iter).size());
+        for(auto const &kw : kw_found) {
+            if (kw.find('*') != std::string::npos) {
+                tokenize_until(cur + kw.size());
             } else {
-                push_token_advance(cur, *iter);
+                push_token_advance(cur, kw);
             }
         }
         cur = cpy;
@@ -1515,12 +1815,20 @@ struct FixedFormRecursiveDescent {
         return next_is(cur, "program");
     }
 
+    bool is_module(unsigned char *cur) {
+        return next_is(cur, "module");
+    }
+
     bool lex_procedure(unsigned char *&cur) {
-        std::vector<std::string> subroutine_keywords{"recursive", "pure",
+	const std::vector<std::string> subroutine_keywords{"recursive", "non_recursive", "pure",
             "elemental"};
-        std::vector<std::string> function_keywords{"recursive", "pure",
-            "elemental",
-            "real*8", "real", "character*1", "character", "complex*16", "complex*8", "complex", "integer", "logical",
+        const std::vector<std::string> function_keywords{"recursive", "non_recursive", "pure",
+            "elemental", "real*", "real",
+	    "character*(*)",
+            "character*", "character",
+            "complex*", "complex",
+            "integer*", "integer",
+            "logical*", "logical",
             "doubleprecision", "doublecomplex"};
         if (is_declaration(cur, "subroutine", subroutine_keywords)) {
             lex_subroutine(cur);
@@ -1533,7 +1841,7 @@ struct FixedFormRecursiveDescent {
         }
     }
 
-    void lex_global_scope_item(unsigned char *&cur) {
+    void lex_global_scope_item(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         // we can define a global assignment
         unsigned char *nline = cur; next_line(nline);
         // eat_label(cur);
@@ -1542,7 +1850,11 @@ struct FixedFormRecursiveDescent {
             tokenize_line(cur);
         }
         if (is_program(cur)) {
-            lex_program(cur, true);
+            lex_program(cur, true, continue_compilation, diagnostics);
+        } else if (next_is(cur, "interface")) {
+            lex_interface(cur);
+        } else if (is_module(cur)) {
+            lex_module(cur);
         } else if (lex_procedure(cur)) {
             return;
         } else if (next_is(cur, "blockdata")) {
@@ -1552,18 +1864,18 @@ struct FixedFormRecursiveDescent {
             push_token_no_advance(cur, "program");
             push_token_no_advance_token(cur, "implicit_program_lfortran", TK_NAME);
             push_token_no_advance(cur, "\n");
-            lex_program(cur, false);
+            lex_program(cur, false, continue_compilation, diagnostics);
         } else {
             error(cur, "ICE: Cannot recognize global scope entity");
         }
     }
 
-    void lex_global_scope(unsigned char *&cur) {
+    void lex_global_scope(unsigned char *&cur, bool continue_compilation, diag::Diagnostics &diagnostics) {
         auto next = cur;
         while (*cur != '\0') {
             // eat_label(cur);
             next_line(next);
-            lex_global_scope_item(cur);
+            lex_global_scope_item(cur, continue_compilation, diagnostics);
             next = cur;
         }
         push_token_no_advance(cur, "EOF");
@@ -1571,7 +1883,7 @@ struct FixedFormRecursiveDescent {
 
 };
 
-bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al) {
+bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocator &al, bool continue_compilation) {
     // We use a recursive descent parser.  We are starting at the global scope
     try {
         FixedFormRecursiveDescent f(diagnostics, al);
@@ -1581,22 +1893,21 @@ bool FixedFormTokenizer::tokenize_input(diag::Diagnostics &diagnostics, Allocato
         f.t.string_start = string_start;
         f.t.cur_line = string_start;
         f.t.line_num = 1;
-        f.lex_global_scope(cur);
+        f.lex_global_scope(cur, continue_compilation, diagnostics);
         tokens = std::move(f.tokens);
         stypes = std::move(f.stypes);
         locations = std::move(f.locations);
         LCOMPILERS_ASSERT(tokens.size() == stypes.size())
         LCOMPILERS_ASSERT(tokens.size() == locations.size())
         tokenized = true;
-    } catch (const parser_local::TokenizerError &e) {
-        diagnostics.diagnostics.push_back(e.d);
+    } catch (const parser_local::TokenizerAbort &) {
         return false;
     }
     return true;
 }
 
 int FixedFormTokenizer::lex(Allocator &/*al*/, YYSTYPE &yylval,
-        Location &loc, diag::Diagnostics &/*diagnostics*/)
+        Location &loc, diag::Diagnostics &/*diagnostics*/, bool /*continue_compilation*/)
 {
     if (!tokens.empty()) {
         auto tok = tokens[token_pos];
