@@ -24,6 +24,8 @@ namespace LCompilers {
 
         llvm::Value* lfortran_malloc(llvm::LLVMContext &context, llvm::Module &module,
                 llvm::IRBuilder<> &builder, llvm::Value* arg_size);
+        llvm::Value* lfortran_realloc(llvm::LLVMContext &context, llvm::Module &module,
+                llvm::IRBuilder<> &builder, llvm::Value* ptr, llvm::Value* arg_size);
 
         /*
         * This function checks whether the
@@ -75,8 +77,7 @@ namespace LCompilers {
                     llvm::IRBuilder<>* builder,
                     LLVMUtils* llvm_utils,
                     DESCR_TYPE descr_type,
-                    CompilerOptions& co_,
-                    std::vector<llvm::Value*>& heap_arrays_);
+                    CompilerOptions& co_);
 
                 /*
                 * Checks whether the given ASR::ttype_t* is an
@@ -116,6 +117,7 @@ namespace LCompilers {
                 */
                 virtual
                 llvm::Type* get_array_type(
+                    ASR::expr_t* expr,
                     ASR::ttype_t* m_type_,
                     llvm::Type* el_type,
                     bool get_pointer=false) = 0;
@@ -134,14 +136,14 @@ namespace LCompilers {
                 */
                 virtual
                 void fill_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
+                    llvm::Type* arr_ty, llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
                     std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
                     llvm::Module* module, bool reserve_data_memory=true) = 0;
 
                 virtual
-                void fill_array_details(
+                void fill_array_details(ASR::expr_t* src_expr, ASR::expr_t* dest_expr,
                     llvm::Value* source, llvm::Value* destination,
-                    ASR::ttype_t* asr_shape_type, bool ignore_data) = 0;
+                    ASR::ttype_t* source_array_type, ASR::ttype_t* dest_array_type, llvm::Module* module, bool ignore_data) = 0;
 
                 /*
                 * Fills the elements of the input array descriptor
@@ -149,31 +151,38 @@ namespace LCompilers {
                 */
                 virtual
                 void fill_malloc_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
-                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
+                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, ASR::ttype_t* asr_type, int n_dims,
+                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims, llvm::Value* string_len,
                     llvm::Module* module, bool realloc=false) = 0;
 
                 virtual
-                void fill_dimension_descriptor(
-                    llvm::Value* arr, int n_dims, llvm::Module* module,ASR::ttype_t* type) = 0;
+                void fill_dimension_descriptor(llvm::Type* type,llvm::Value* arr, int n_dims) = 0;
 
                 virtual
                 void reset_array_details(
-                    llvm::Value* arr, llvm::Value* source_arr, int n_dims) = 0;
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr, int n_dims) = 0;
+
+                virtual
+                void reset_array_details(
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr,
+                    llvm::Value** lbs, llvm::Value** lengths,
+                    int n_dims) = 0;
 
                 virtual
                 void fill_descriptor_for_array_section(
-                    llvm::Value* value_desc, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
-                    int value_rank, int target_rank) = 0;
+                    int value_rank, int target_rank, LocationManager& lm) = 0;
 
                 virtual
                 void fill_descriptor_for_array_section_data_only(
-                    llvm::Value* value_desc, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
-                    llvm::Value** llvm_diminfo, int value_rank, int target_rank) = 0;
+                    llvm::Value** llvm_diminfo, int value_rank, int target_rank, LocationManager& lm) = 0;
 
                 /*
                 * Returns the llvm::Type* associated with the
@@ -188,7 +197,16 @@ namespace LCompilers {
                 * implemented by current class.
                 */
                 virtual
-                llvm::Value* get_pointer_to_data(llvm::Value* arr) = 0;
+                llvm::Value* get_pointer_to_data(llvm::Type* type, llvm::Value* arr) = 0;
+
+                /*
+                * Returns pointer to data in the input
+                * array descriptor according to the rules
+                * implemented by current class.
+                * Uses ASR type to get the corresponding LLVM type
+                */
+                virtual
+                llvm::Value* get_pointer_to_data(ASR::expr_t* arr_expr, ASR::ttype_t* arr_type, llvm::Value* arr, llvm::Module* module) = 0;
 
                 /*
                 * Returns offset in the input
@@ -196,7 +214,7 @@ namespace LCompilers {
                 * implemented by current class).
                 */
                 virtual
-                llvm::Value* get_offset(llvm::Value* dim_des, bool load=true) = 0;
+                llvm::Value* get_offset(llvm::Type* type, llvm::Value* dim_des, bool load=true) = 0;
 
                 /*
                 * Returns lower bound in the input
@@ -232,14 +250,14 @@ namespace LCompilers {
                     llvm::Value* dim, bool load=true) = 0;
 
                 virtual
-                llvm::Value* get_dimension_size(llvm::Value* dim_des,
+                llvm::Value* get_dimension_size(llvm::Value* dim_des_arr,
                     bool load=true) = 0;
 
                 virtual
-                llvm::Value* get_rank(llvm::Value* arr, bool get_pointer=false) = 0;
+                llvm::Value* get_rank(llvm::Type* type, llvm::Value* arr, bool get_pointer=false) = 0;
 
                 virtual
-                void set_rank(llvm::Value* arr, llvm::Value* rank) = 0;
+                void set_rank(llvm::Type* type, llvm::Value* arr, llvm::Value* rank) = 0;
 
                 /*
                 * Returns pointer to dimension descriptor array
@@ -249,8 +267,6 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Type *type, llvm::Value* arr, bool load=true) = 0;
 
-                virtual
-                llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Value* arr, bool load=true) = 0;
 
                 /*
                 * Returns pointer to the dimension descriptor
@@ -269,36 +285,61 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_single_element(llvm::Type *type, llvm::Value* array,
                     std::vector<llvm::Value*>& m_args, int n_args,
+                    ASR::ttype_t* asr_type, ASR::expr_t* expr, LocationManager& lm,
                     bool data_only=false, bool is_fixed_size=false,
                     llvm::Value** llvm_diminfo=nullptr,
-                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr, bool is_unbounded_pointer_to_data = false) = 0;
+                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr,
+                    bool is_unbounded_pointer_to_data = false, bool check_for_bounds = false, std::string array_name = "", std::string infile = "") = 0;
 
                 virtual
-                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type) = 0;
+                llvm::Value* get_is_allocated_flag(llvm::Value* array, ASR::expr_t* array_exp) = 0;
 
                 virtual
-                void reset_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type) = 0;
+                void reset_is_allocated_flag(llvm::Type* tmp_typ, llvm::Value* array, llvm::Type* llvm_data_type) = 0;
 
 
                 virtual
-                llvm::Value* reshape(llvm::Value* array, llvm::Type* llvm_data_type,
-                                     llvm::Value* shape, ASR::ttype_t* asr_shape_type,
+                llvm::Value* reshape(llvm::Type* arr_type, llvm::Value* array, llvm::Type* llvm_data_type,
+                                     llvm::Type* shape_type, llvm::Value* shape, ASR::ttype_t* asr_shape_type,
                                      llvm::Module* module) = 0;
 
                 virtual
-                void copy_array(llvm::Value* src, llvm::Value* dest,
+                void copy_array(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
                                 llvm::Module* module, ASR::ttype_t* asr_data_type,
-                                bool create_dim_des_array, bool reserve_memory) = 0;
+                                bool reserve_memory) = 0;
+
+                virtual
+                void copy_array_move_allocation(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
+                                llvm::Module* module, ASR::expr_t* array_exp, ASR::ttype_t* asr_data_type) = 0;
 
                 virtual
                 void copy_array_data_only(llvm::Value* src, llvm::Value* dest,
                                           llvm::Module* module, llvm::Type* llvm_data_type,
+                                          ASR::ttype_t* arr_type,
                                           llvm::Value* num_elements) = 0;
 
                 virtual
-                llvm::Value* get_array_size(llvm::Value* array, llvm::Value* dim,
+                llvm::Value* get_array_size(llvm::Type* type, llvm::Value* array, llvm::Value* dim,
                                             int output_kind, int dim_kind=4) = 0;
 
+                /*
+                * Creates a contiguous copy of data from a DescriptorArray.
+                * Helpful when having strided access to an array.
+                * Returns a pointer to the newly allocated contiguous buffer.
+                * Allocation is done on heap using lfortran_malloc,
+                * So, please make sure to free the memory after use.
+                *
+                * Note: Similar logic exists in ASR passes (pass/print_arr.cpp & pass/array_op.cpp).
+                * But, the ASR pass for print statements was disabled in #6351.
+                * Enabling it opened other issues, hence this separate LLVM implementation
+                * to handle strided arrays for writing array into strings (used in print statements), 
+                * by storing in a contiguous buffer first.
+                */
+                virtual
+                llvm::Value* create_contiguous_copy_from_descriptor(
+                    llvm::Type* source_llvm_type, llvm::Value* source_desc,
+                    llvm::Type* elem_type, int rank, llvm::Module* module) = 0;
+
         };
 
         class SimpleCMODescriptor: public Descriptor {
@@ -314,22 +355,20 @@ namespace LCompilers {
                 std::map<std::string, std::pair<llvm::StructType*, llvm::Type*>> tkr2array;
 
                 CompilerOptions& co;
-                std::vector<llvm::Value*>& heap_arrays;
 
                 llvm::Value* cmo_convertor_single_element(
-                    llvm::Value* arr, std::vector<llvm::Value*>& m_args,
-                    int n_args, bool check_for_bounds);
+                    llvm::Type* type, llvm::Value* arr, std::vector<llvm::Value*>& m_args,
+                    int n_args, bool check_for_bounds, LocationManager& lm, std::string array_name = "", std::string infile = "", Location loc = {0, 0});
 
                 llvm::Value* cmo_convertor_single_element_data_only(
                     llvm::Value** llvm_diminfo, std::vector<llvm::Value*>& m_args,
-                    int n_args, bool check_for_bounds, bool is_unbounded_pointer_to_data = false);
+                    int n_args, bool check_for_bounds, LocationManager& lm, bool is_unbounded_pointer_to_data = false, std::string array_name = "", std::string infile = "", Location loc = {0, 0});
 
             public:
 
                 SimpleCMODescriptor(llvm::LLVMContext& _context,
                     llvm::IRBuilder<>* _builder,
-                    LLVMUtils* _llvm_utils, CompilerOptions& co_,
-                    std::vector<llvm::Value*>& heap_arrays);
+                    LLVMUtils* _llvm_utils, CompilerOptions& co_);
 
                 virtual
                 bool is_array(ASR::ttype_t* asr_type);
@@ -349,6 +388,7 @@ namespace LCompilers {
 
                 virtual
                 llvm::Type* get_array_type(
+                    ASR::expr_t* expr,
                     ASR::ttype_t* m_type_,
                     llvm::Type* el_type,
                     bool get_pointer=false);
@@ -358,57 +398,71 @@ namespace LCompilers {
 
                 virtual
                 void fill_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
+                    llvm::Type* arr_ty, llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
                     std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
                     llvm::Module* module, bool reserve_data_memory=true);
 
                 virtual
-                void fill_array_details(
+                void fill_array_details(ASR::expr_t* src_expr, ASR::expr_t* dest_expr,
                     llvm::Value* source, llvm::Value* destination,
-                    ASR::ttype_t* asr_shape_type, bool ignore_data);
+                    ASR::ttype_t* source_array_type, ASR::ttype_t* dest_array_type, llvm::Module* module, bool ignore_data);
 
                 virtual
                 void fill_malloc_array_details(
-                    llvm::Value* arr, llvm::Type* llvm_data_type, int n_dims,
-                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims,
+                    llvm::Value* arr, llvm::Type *arr_type, llvm::Type* llvm_data_type, ASR::ttype_t* asr_type, int n_dims,
+                    std::vector<std::pair<llvm::Value*, llvm::Value*>>& llvm_dims, llvm::Value* string_len,
                     llvm::Module* module, bool realloc=false);
 
                 virtual
-                void fill_dimension_descriptor(
-                    llvm::Value* arr, int n_dims, llvm::Module* module, ASR::ttype_t* type);
+                void fill_dimension_descriptor(llvm::Type* type, llvm::Value* arr, int n_dims);
 
                 virtual
                 void reset_array_details(
-                    llvm::Value* arr, llvm::Value* source_arr, int n_dims);
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr, int n_dims);
+
+                virtual
+                void reset_array_details(
+                    llvm::Type* type, llvm::Value* arr, llvm::Value* source_arr,
+                    llvm::Value** lbs, llvm::Value** lengths,
+                    int n_dims);
 
                 virtual
                 void fill_descriptor_for_array_section(
-                    llvm::Value* value_desc, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
-                    int value_rank, int target_rank);
+                    int value_rank, int target_rank, LocationManager& lm);
 
                 virtual
                 void fill_descriptor_for_array_section_data_only(
-                    llvm::Value* value_desc, llvm::Value* target,
+                    llvm::Value* value_desc, llvm::Type* value_el_type, ASR::ttype_t* value_type,
+                    llvm::Value* target, ASR::ttype_t* target_type, ASR::expr_t* target_expr,
                     llvm::Value** lbs, llvm::Value** ubs,
                     llvm::Value** ds, llvm::Value** non_sliced_indices,
-                    llvm::Value** llvm_diminfo, int value_rank, int target_rank);
+                    llvm::Value** llvm_diminfo, int value_rank, int target_rank, LocationManager& lm);
 
                 virtual
                 llvm::Type* get_dimension_descriptor_type(bool get_pointer=false);
 
                 virtual
-                llvm::Value* get_pointer_to_data(llvm::Value* arr);
+                llvm::Value* get_pointer_to_data(llvm::Type* type, llvm::Value* arr);
+                
+                /*
+                 * Return pointer to data in array descriptor,
+                 * Used arr_type to get the corresponding llvm::Type (LLVM 17+).
+                */
+                virtual
+                llvm::Value* get_pointer_to_data(ASR::expr_t* arr_expr, ASR::ttype_t* arr_type, llvm::Value* arr, llvm::Module* module);
 
                 virtual
-                llvm::Value* get_rank(llvm::Value* arr, bool get_pointer=false);
+                llvm::Value* get_rank(llvm::Type* type, llvm::Value* arr, bool get_pointer=false);
 
                 virtual
-                void set_rank(llvm::Value* arr, llvm::Value* rank);
+                void set_rank(llvm::Type* type, llvm::Value* arr, llvm::Value* rank);
 
                 virtual
-                llvm::Value* get_offset(llvm::Value* dim_des, bool load=true);
+                llvm::Value* get_offset(llvm::Type* type, llvm::Value* dim_des, bool load=true);
 
                 virtual
                 llvm::Value* get_lower_bound(llvm::Value* dim_des, bool load=true);
@@ -421,15 +475,12 @@ namespace LCompilers {
                     llvm::Value* dim, bool load=true);
 
                 virtual
-                llvm::Value* get_dimension_size(llvm::Value* dim_des,
+                llvm::Value* get_dimension_size(llvm::Value* dim_des_arr,
                     bool load=true);
 
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Type* type, llvm::Value* arr, bool load=true);
 
-                virtual
-                llvm::Value* get_pointer_to_dimension_descriptor_array(llvm::Value* arr, bool load=true);
-
                 virtual
                 llvm::Value* get_pointer_to_dimension_descriptor(llvm::Value* dim_des_arr,
                     llvm::Value* dim);
@@ -440,35 +491,47 @@ namespace LCompilers {
                 virtual
                 llvm::Value* get_single_element(llvm::Type *type, llvm::Value* array,
                     std::vector<llvm::Value*>& m_args, int n_args,
+                    ASR::ttype_t* asr_type, ASR::expr_t* expr, LocationManager& lm,
                     bool data_only=false, bool is_fixed_size=false,
                     llvm::Value** llvm_diminfo=nullptr,
-                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr, bool is_unbounded_pointer_to_data = false);
+                    bool polymorphic=false, llvm::Type* polymorphic_type=nullptr,
+                    bool is_unbounded_pointer_to_data = false, bool check_for_bounds = false, std::string array_name = "", std::string infile = "");
 
                 virtual
-                llvm::Value* get_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type);
+                llvm::Value* get_is_allocated_flag(llvm::Value* array, ASR::expr_t* array_exp);
 
                 virtual
-                void reset_is_allocated_flag(llvm::Value* array, llvm::Type* llvm_data_type);
+                void reset_is_allocated_flag(llvm::Type* typ_tmp, llvm::Value* array, llvm::Type* llvm_data_type);
 
                 virtual
-                llvm::Value* reshape(llvm::Value* array, llvm::Type* llvm_data_type,
-                                     llvm::Value* shape, ASR::ttype_t* asr_shape_type,
+                llvm::Value* reshape(llvm::Type* arr_type, llvm::Value* array, llvm::Type* llvm_data_type,
+                                     llvm::Type* shape_type, llvm::Value* shape, ASR::ttype_t* asr_shape_type,
                                      llvm::Module* module);
 
                 virtual
-                void copy_array(llvm::Value* src, llvm::Value* dest,
+                void copy_array(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
                                 llvm::Module* module, ASR::ttype_t* asr_data_type,
-                                bool create_dim_des_array, bool reserve_memory);
+                                bool reserve_memory);
+
+                virtual
+                void copy_array_move_allocation(llvm::Type* src_ty, llvm::Value* src, llvm::Type* dest_ty, llvm::Value* dest,
+                                llvm::Module* module, ASR::expr_t* array_exp, ASR::ttype_t* asr_data_type);
 
                 virtual
                 void copy_array_data_only(llvm::Value* src, llvm::Value* dest,
                                           llvm::Module* module, llvm::Type* llvm_data_type,
+                                          ASR::ttype_t* arr_type,
                                           llvm::Value* num_elements);
 
                 virtual
-                llvm::Value* get_array_size(llvm::Value* array, llvm::Value* dim,
+                llvm::Value* get_array_size(llvm::Type* type, llvm::Value* array, llvm::Value* dim,
                                             int output_kind, int dim_kind=4);
 
+                virtual
+                llvm::Value* create_contiguous_copy_from_descriptor(
+                    llvm::Type* source_llvm_type, llvm::Value* source_desc,
+                    llvm::Type* elem_type, int rank, llvm::Module* module);
+
         };
 
     } // LLVMArrUtils
