@@ -29,7 +29,7 @@ intrinsic_funcs_args = {
     "Mod": [
         {
             "args": [("int", "int"), ("real", "real")],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": "dynamic"
         },
     ],
     "Trailz": [
@@ -62,18 +62,6 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 1
         },
     ],
-    "Mvbits": [
-        {
-            "args": [("int", "int", "int", "int", "int")],
-            "ret_type_arg_idx": 3
-        },
-    ],
-    "MoveAlloc": [
-        {
-            "args": [("any", "any")],
-            "ret_type_arg_idx": 0
-        },
-    ],
     "Leadz": [
         {
             "args": [("int",)],
@@ -183,10 +171,16 @@ intrinsic_funcs_args = {
             "return": "int32"
         }
     ],
+    "SelectedLogicalKind": [
+        {
+            "args": [("int",)],
+            "return": "int32"
+        }
+    ],
     "Logical": [
         {
             "args": [("bool", )],
-            "ret_type_arg_idx": 0, 
+            "ret_type_arg_idx": 0,
             "kind_arg": True
         }
     ],
@@ -196,12 +190,6 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
-    "Repeat": [
-        {
-            "args": [("char", "int")],
-            "ret_type_arg_idx": 0
-        }
-    ],
     "StringContainsSet": [
         {
             "args": [("char", "char", "bool", "int")],
@@ -263,12 +251,6 @@ intrinsic_funcs_args = {
             "return": "int32"
         },
     ],
-    "IsContiguous": [
-        {
-            "args": [("any",)],
-            "return": "logical"
-        },
-    ],
     "OutOfRange": [
         {
             "args": [("int", "real", "bool"), ("real", "real", "bool"), ("int", "int", "bool"), ("real", "int", "bool")],
@@ -291,13 +273,15 @@ intrinsic_funcs_args = {
     "Adjustl": [
         {
             "args": [("char",)],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "allow_polymorphic_arg": [0]
         }
     ],
     "Adjustr": [
         {
             "args": [("char",)],
-            "ret_type_arg_idx": 0
+            "ret_type_arg_idx": 0,
+            "allow_polymorphic_arg": [0]
         }
     ],
     "Aint": [
@@ -326,6 +310,13 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Idnint": [
+        {
+            "args": [("real",)],
+            "return": "int32",
+            "kind_validation": [{"first":{0: 8}}]
+        }
+    ],
     "Anint": [
         {
             "args": [("real",)],
@@ -506,12 +497,14 @@ intrinsic_funcs_args = {
     "Ifix": [
         {
             "args": [("real",)],
-            "return": "int32"
+            "return": "int32",
+            "kind_validation": [{"first":{0: 4}}]
         }
     ],
     "Idint": [
         {
             "args": [("real",)],
+            "kind_validation": [{"first":{0: 8}}],
             "return": "int32"
         }
     ],
@@ -582,6 +575,12 @@ intrinsic_funcs_args = {
             "same_kind_arg": 2
         },
     ],
+    "And": [
+        {
+            "args": [("int", "int"),("bool","bool")],
+            "ret_type_arg_idx": 0,
+        },
+    ],
     "Ior": [
         {
             "args": [("int", "int")],
@@ -589,6 +588,12 @@ intrinsic_funcs_args = {
             "same_kind_arg": 2
         },
     ],
+    "Or": [
+        {
+            "args": [("int", "int"), ("bool", "bool")],
+            "ret_type_arg_idx": 0,
+        },
+    ],
     "Ieor": [
         {
             "args": [("int", "int")],
@@ -596,6 +601,12 @@ intrinsic_funcs_args = {
             "same_kind_arg": 2
         },
     ],
+    "Xor": [
+        {
+            "args": [("int", "int"), ("bool", "bool")],
+            "ret_type_arg_idx": 0,
+        },
+    ],
     "Ibclr": [
         {
             "args": [("int", "int")],
@@ -651,12 +662,6 @@ intrinsic_funcs_args = {
             "return": "real64",
         },
     ],
-    "Rank": [
-        {
-            "args": [("any",)],
-            "return": "int32"
-        }
-    ],
     "BitSize": [
         {
             "args": [("int",)],
@@ -666,7 +671,7 @@ intrinsic_funcs_args = {
     "NewLine": [
         {
             "args": [("char",)],
-            "return": "character(-1)"
+            "return": "character(1)"
         }
     ],
     "Range": [
@@ -711,9 +716,16 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0
         }
     ],
+    "Loc": [
+        {
+            "args": [("any",)],
+            "return": "int32"
+        }
+    ],
     "Dprod": [
         {
             "args": [("real", "real")],
+            "kind_validation": [{"first":{0: 4}}, {"second":{1: 4}}],
             "return": "real64"
         }
     ],
@@ -749,7 +761,7 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0,
             "same_kind_arg": 3
         }
-    ], 
+    ],
     "Ishftc": [
         {
             "args": [("int", "int", "int")],
@@ -760,7 +772,10 @@ intrinsic_funcs_args = {
         {
             "args": [("char",)],
             "return": "int32",
-            "kind_arg": True
+            "kind_arg": True,
+            "char_len_validation": [
+                {1: {0: {"min": 1, "max": 1}}}
+            ]
         },
     ],
     "Char": [
@@ -777,6 +792,16 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Iachar": [
+        {
+            "args": [("char",)],
+            "return": "int32",
+            "kind_arg": True,
+            "char_len_validation": [
+                {1: {0: {"min": 1, "max": 1}}}
+            ]
+        },
+    ],
     "Exponent": [
         {
             "args": [("real",)],
@@ -831,6 +856,7 @@ intrinsic_funcs_args = {
             "args": [("int",), ("real",), ("complex",)],
             "return": "real32",
             "kind_arg": True,
+            "real_32_except_complex": True
         },
     ],
     "Int": [
@@ -844,13 +870,15 @@ intrinsic_funcs_args = {
         {
             "args": [("char",)],
             "return": "int32",
-            "kind_arg": True
+            "kind_arg": True,
+            "allow_polymorphic_arg": [0]
         }
     ],
     "StringTrim": [
         {
             "args": [("char",)],
-            "ret_type_arg_idx": 0
+            "return" : "allocatable_deferred_string()",
+            "allow_polymorphic_arg": [0]
         }
     ],
 }
@@ -863,9 +891,9 @@ compile_time_only_fn = [
     "StorageSize",
     "Range",
     "Precision",
-    "Rank",
     "Tiny",
     "Huge",
+    "Loc",
     "BitSize",
     "NewLine",
     "Kind",
@@ -910,7 +938,7 @@ def compute_arg_kinds(indent, no_of_args):
     for i in range(no_of_args):
         src += indent + f"int kind{i} = ASRUtils::extract_kind_from_ttype_t(arg_type{i});\n"
 
-def compute_arg_condition(no_of_args, args_lists):
+def compute_arg_condition(no_of_args, args_lists, allow_polymorphic_arg, args_var):
     condition = []
     cond_in_msg = []
     for arg_list in args_lists:
@@ -918,7 +946,11 @@ def compute_arg_condition(no_of_args, args_lists):
         subcond_in_msg = []
         for i in range(no_of_args):
             arg = arg_list[i]
-            subcond.append(f"{type_to_asr_type_check[arg]}(*arg_type{i})")
+            type_check = f"{type_to_asr_type_check[arg]}(*arg_type{i})"
+            # Add unlimited polymorphic check if specified
+            if allow_polymorphic_arg and i in allow_polymorphic_arg:
+                type_check = f"({type_check} || (is_unlimited_polymorphic_type({args_var}[{i}])))"
+            subcond.append(type_check)
             subcond_in_msg.append(arg)
         condition.append(" && ".join(subcond))
         cond_in_msg.append(", ".join(subcond_in_msg))
@@ -944,7 +976,8 @@ def add_verify_arg_type_src(func_name):
         src += 2 * indent + f"{else_if} (x.n_args == {no_of_args}) " + " {\n"
         src += 3 * indent + f'ASRUtils::require_impl(x.m_overload_id == {i}, "Overload Id for {func_name} expected to be {i}, found " + std::to_string(x.m_overload_id), x.base.base.loc, diagnostics);\n'
         compute_arg_types(3 * indent, no_of_args, "x.m_args")
-        condition, cond_in_msg = compute_arg_condition(no_of_args, args_lists)
+        allow_polymorphic_arg = arg_info.get("allow_polymorphic_arg", None)
+        condition, cond_in_msg = compute_arg_condition(no_of_args, args_lists, allow_polymorphic_arg, "x.m_args")
         src += 3 * indent + f'ASRUtils::require_impl({condition}, "Unexpected args, {func_name} expects {cond_in_msg} as arguments", x.base.base.loc, diagnostics);\n'
         if same_kind_arg:
             compute_arg_kinds(3 * indent, same_kind_arg)
@@ -986,7 +1019,8 @@ def add_create_func_arg_type_src(func_name):
         else_if = "else if" if i > 0 else "if"
         src += 2 * indent + f"{else_if} (args.size() == {no_of_args + int(kind_arg)}) " + " {\n"
         compute_arg_types(3 * indent, no_of_args, "args")
-        condition, cond_in_msg = compute_arg_condition(no_of_args, args_lists)
+        allow_polymorphic_arg = arg_info.get("allow_polymorphic_arg", None)
+        condition, cond_in_msg = compute_arg_condition(no_of_args, args_lists, allow_polymorphic_arg, "args")
         src += 3 * indent + f'if(!({condition}))' + ' {\n'
         src += 4 * indent + f'append_error(diag, "Unexpected args, {func_name} expects {cond_in_msg} as arguments", loc);\n'
         src += 4 * indent + f'return nullptr;\n'
@@ -998,6 +1032,34 @@ def add_create_func_arg_type_src(func_name):
             src += 4 * indent + f'append_error(diag, "Kind of all the arguments of {func_name} must be the same", loc);\n'
             src += 4 * indent + f'return nullptr;\n'
             src += 3 * indent + '}\n'
+        kind_validation_info = arg_info.get("kind_validation", [])
+        if kind_validation_info != []:
+            src += 3 * indent + "int kind = 0;\n"
+        for validation_item in kind_validation_info:
+            for arg_name, arg_spec in validation_item.items():
+                arg_pos = list(arg_spec.keys())[0]
+                required_kind = list(arg_spec.values())[0]
+                src += 3 * indent + f"kind = ASRUtils::extract_kind_from_ttype_t(expr_type(args[{arg_pos}]));\n"
+                src += 3 * indent + f"if(kind != {required_kind}) " + "{\n"
+                src += 4 * indent + f'append_error(diag, "{arg_name} argument of `{func_name.lower()}` must have kind equal to {required_kind}", loc);\n'
+                src += 4 * indent + f'return nullptr;\n'
+                src += 3 * indent + '}\n'
+        char_len_validation_info = arg_info.get("char_len_validation", [])
+        for validation_item in char_len_validation_info:
+            for arg_name, arg_spec in validation_item.items():
+                arg_pos = list(arg_spec.keys())[0]
+                constraint = list(arg_spec.values())[0]
+                src += 3 * indent + f'if (ASR::is_a<ASR::String_t>(*arg_type{arg_pos})) {{\n'
+                src += 4 * indent + f'int64_t len = ASRUtils::get_fixed_string_len(arg_type{arg_pos});\n'
+                if constraint["min"] == constraint["max"]:
+                    src += 4 * indent + f'if (len != -1 && len != {constraint["min"]}) {{\n'
+                    src += 5 * indent + f'append_error(diag, "Argument {arg_name} to {func_name} must have length {constraint["min"]}", loc);\n'
+                else:
+                    src += 4 * indent + f'if (len != -1 && (len < {constraint["min"]} || len > {constraint["max"]})) {{\n'
+                    src += 5 * indent + f'append_error(diag, "Argument {arg_name} to {func_name} must have length between {constraint["min"]} and {constraint["max"]}", loc);\n'
+                src += 5 * indent + 'return nullptr;\n'
+                src += 4 * indent + '}\n'
+                src += 3 * indent + '}\n'
         src += 2 * indent + "}\n"
     src += 2 * indent + "else {\n"
     src += 3 * indent + f'append_error(diag, "Unexpected number of args, {func_name} takes {no_of_args_msg} arguments, found " + std::to_string(args.size()), loc);\n'
@@ -1017,7 +1079,16 @@ def add_create_func_return_src(func_name):
     else:
         src += indent * 2 + "ASRUtils::ExprStmtDuplicator expr_duplicator(al);\n"
         src += indent * 2 + "expr_duplicator.allow_procedure_calls = true;\n"
-        src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(expr_type(args[{ret_type_arg_idx}]));\n"
+        if ( ret_type_arg_idx == "dynamic"):
+            src += indent * 2 + f"int upper_kind = 0;\n"
+            src += indent * 2 + f"for(size_t i=0;i<args.size();i++){{\n"
+            src += indent * 3 + f"upper_kind = std::max(upper_kind,ASRUtils::extract_kind_from_ttype_t(expr_type(args[i])));\n"
+            src += indent * 2 + f"}}\n"
+            src += indent * 2 + f"ASR::ttype_t* type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[0])));\n"
+            src += indent * 2 + f"set_kind_to_ttype_t(type_,upper_kind);\n"
+        else:
+            src += indent * 2 + "ASR::ttype_t* type_ = nullptr;\n"
+            src += indent * 2 + f"type_ = expr_duplicator.duplicate_ttype(ASRUtils::extract_type(expr_type(args[{ret_type_arg_idx}])));\n"
         ret_type = "type_"
     kind_arg = arg_infos[0].get("kind_arg", False)
     src += indent * 2 + f"ASR::ttype_t *return_type = {ret_type};\n"
@@ -1029,7 +1100,16 @@ def add_create_func_return_src(func_name):
         src += indent * 4 +         "return nullptr;\n"
         src += indent * 3 +     "}\n"
         src += indent * 3 +     "set_kind_to_ttype_t(return_type, kind);\n"
-        src += indent * 2 + "}\n"        
+        src += indent * 2 + "}\n"
+    real_32_except_complex = arg_infos[0].get("real_32_except_complex", False)
+    if real_32_except_complex:
+        src += indent * 2 + "else { \n"
+        src += indent * 3 + "ASR::ttype_t* arg_type = ASRUtils::expr_type(args[0]);\n"
+        src += indent * 3 + "if (is_complex(*arg_type)) { \n"
+        src += indent * 4 + "int kind = ASRUtils::extract_kind_from_ttype_t(arg_type); \n"
+        src += indent * 4 + "set_kind_to_ttype_t(return_type, kind); \n"
+        src += indent * 3 + "} \n"
+        src += indent * 2 + "} \n"
     src += indent * 2 + "ASR::expr_t *m_value = nullptr;\n"
     src += indent * 2 + f"Vec<ASR::expr_t*> m_args; m_args.reserve(al, {no_of_args});\n"
     for _i in range(no_of_args):
@@ -1045,12 +1125,18 @@ def add_create_func_return_src(func_name):
             "ASRUtils::expr_type(m_args[0]), m_args[0], return_type, m_value);\n"
 
     else:
-        if ret_type_val:
-            src += indent * 2 + "ASR::ttype_t* type = ASRUtils::expr_type(args[0]);\n"
-            src += indent * 2 + "if (ASR::is_a<ASR::Array_t>(*type)) {\n"
-            src += indent * 3 + "ASR::Array_t* e = ASR::down_cast<ASR::Array_t>(type);\n"
-            src += indent * 3 + f"return_type = TYPE(ASR::make_Array_t(al, type->base.loc,  return_type, e->m_dims, e->n_dims, ASR::array_physical_typeType::FixedSizeArray));\n"
-            src += indent * 2 + "}\n"
+        src += indent * 2 +     f"for( size_t i = 0; i < {no_of_args}; i++ ) " + "{\n"
+        src += indent * 3 +         "ASR::ttype_t* type = ASRUtils::expr_type(args[i]);\n"
+        src += indent * 3 +         "if (ASRUtils::is_array(type)) {\n"
+        src += indent * 4 +             "ASR::dimension_t* m_dims = nullptr;\n"
+        src += indent * 4 +             "size_t n_dims = ASRUtils::extract_dimensions_from_ttype(type, m_dims);\n"
+        src += indent * 4 +             "return_type = ASRUtils::make_Array_t_util(al, type->base.loc, "
+        src +=                              "return_type, m_dims, n_dims, ASR::abiType::Source, false, "
+        src +=                              "ASR::array_physical_typeType::DescriptorArray);\n"
+        src += indent * 4 +             "break;\n"
+        src += indent * 3 +         "}\n"
+        src += indent * 2 +     "}\n"
+
         src += indent * 2 + "if (all_args_evaluated(m_args)) {\n"
         src += indent * 3 +     f"Vec<ASR::expr_t*> args_values; args_values.reserve(al, {no_of_args});\n"
         for _i in range(no_of_args):
@@ -1096,9 +1182,15 @@ def get_registry_funcs_src():
     return src
 
 
-HEAD = """#ifndef LIBASR_PASS_INTRINSIC_FUNC_REG_UTIL_H
+HEAD = """// This file is AUTO-GENERATED by src/libasr/intrinsic_func_registry_util_gen.py
+// Do not edit this file directly. Instead, modify intrinsic_func_registry_util_gen.py
+// and regenerate this file by running:
+//     python src/libasr/intrinsic_func_registry_util_gen.py
+
+#ifndef LIBASR_PASS_INTRINSIC_FUNC_REG_UTIL_H
 #define LIBASR_PASS_INTRINSIC_FUNC_REG_UTIL_H
 
+#include <libasr/asr_utils.h>
 #include <libasr/pass/intrinsic_functions.h>
 
 namespace LCompilers {
