@@ -7,8 +7,6 @@
 #include <libasr/pass/pass_utils.h>
 #include <libasr/pass/intrinsic_function_registry.h>
 
-#include <vector>
-#include <utility>
 
 namespace LCompilers {
 
@@ -29,15 +27,128 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
     std::map<ASR::expr_t*, ASR::expr_t*>& resultvar2value;
     bool realloc_lhs, allocate_target;
 
+    ASR::expr_t* get_first_scalar_expr(ASR::expr_t* expr) {
+        // Visitor to replace loop variable with its starting value in an expression
+        class ReplaceLoopVarWithStart : public ASR::BaseExprReplacer<ReplaceLoopVarWithStart> {
+        public:
+            Allocator& al;
+            ASR::symbol_t* loop_var_sym;
+            ASR::expr_t* start_value;
+
+            ReplaceLoopVarWithStart(Allocator& al_, ASR::symbol_t* loop_var_sym_, ASR::expr_t* start_value_)
+                : al(al_), loop_var_sym(loop_var_sym_), start_value(start_value_) {}
+
+            void replace_Var(ASR::Var_t* x) {
+                if (x->m_v == loop_var_sym) {
+                    *current_expr = start_value;
+                }
+            }
+        };
+        if( expr == nullptr ) {
+            return nullptr;
+        }
+        if( ASR::is_a<ASR::ImpliedDoLoop_t>(*expr) ) {
+            ASR::ImpliedDoLoop_t* idl = ASR::down_cast<ASR::ImpliedDoLoop_t>(expr);
+            for( size_t i = 0; i < idl->n_values; i++ ) {
+                // Replace loop variable with its starting value in the expression
+                ASR::expr_t* value_expr = idl->m_values[i];
+                
+                // Get the loop variable symbol
+                if (ASR::is_a<ASR::Var_t>(*idl->m_var)) {
+                    ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(idl->m_var);
+                    ASR::symbol_t* loop_var_sym = var->m_v;
+                    
+                    // Create a copy of the expression and replace the loop var with start value
+                    value_expr = ASRUtils::expr_value(value_expr);
+                    if (value_expr == nullptr) {
+                        value_expr = idl->m_values[i];
+                    }
+
+                    ASRUtils::ExprStmtDuplicator duplicator(al);
+                    value_expr = duplicator.duplicate_expr(value_expr);
+                    
+                    ReplaceLoopVarWithStart replacer(al, loop_var_sym, idl->m_start);
+                    replacer.current_expr = &value_expr;
+                    replacer.replace_expr(value_expr);
+                }
+                
+                ASR::expr_t* candidate = get_first_scalar_expr(value_expr);
+                if( candidate != nullptr ) {
+                    return candidate;
+                }
+            }
+            return nullptr;
+        } else if( ASR::is_a<ASR::ArrayConstructor_t>(*expr) ) {
+            ASR::ArrayConstructor_t* arr_cons = ASR::down_cast<ASR::ArrayConstructor_t>(expr);
+            for( size_t i = 0; i < arr_cons->n_args; i++ ) {
+                ASR::expr_t* candidate = get_first_scalar_expr(arr_cons->m_args[i]);
+                if( candidate != nullptr ) {
+                    return candidate;
+                }
+            }
+            return nullptr;
+        } else if( ASR::is_a<ASR::ArrayConstant_t>(*expr) ) {
+            ASR::ArrayConstant_t* arr_const = ASR::down_cast<ASR::ArrayConstant_t>(expr);
+            int64_t array_size = ASRUtils::get_fixed_size_of_array(arr_const->m_type);
+            if( array_size <= 0 ) {
+                return nullptr;
+            }
+            return ASRUtils::fetch_ArrayConstant_value(al, arr_const, 0);
+        }
+        return expr;
+    }
+
+    ASR::expr_t* get_string_source_len(ASR::expr_t* expr) {
+        // Derive a conservative element length without evaluating indices (e.g., trim(lines(iii)) uses declared length).
+        if (!expr) return nullptr;
+        if (ASR::is_a<ASR::IntrinsicElementalFunction_t>(*expr)) {
+            ASR::IntrinsicElementalFunction_t* f = ASR::down_cast<ASR::IntrinsicElementalFunction_t>(expr);
+            if (f->m_intrinsic_id == static_cast<int64_t>(ASRUtils::IntrinsicElementalFunctions::StringTrim) && f->n_args >= 1) {
+                return get_string_source_len(f->m_args[0]);
+            } else if (f->m_intrinsic_id == static_cast<int64_t>(ASRUtils::IntrinsicElementalFunctions::StringConcat) && f->n_args > 1) {
+                ASR::expr_t* total_len = nullptr;
+                for (size_t i = 0; i < f->n_args; i++) {
+                    ASR::expr_t* arg_len = get_string_source_len(f->m_args[i]);
+                    if (arg_len == nullptr) {
+                        return nullptr;
+                    }
+                    if (total_len == nullptr) {
+                        total_len = arg_len;
+                    } else {
+                        ASR::ttype_t* int_type = ASRUtils::TYPE(ASR::make_Integer_t(al, expr->base.loc, 4));
+                        total_len = ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, expr->base.loc,
+                            total_len, ASR::binopType::Add, arg_len,
+                            int_type, nullptr));
+                    }
+                }
+                return total_len;
+            }
+        }
+
+        ASR::ttype_t* t = ASRUtils::expr_type(expr);
+        t = ASRUtils::extract_type(t);
+        if (ASR::is_a<ASR::String_t>(*t)) {
+            ASR::String_t* st = ASR::down_cast<ASR::String_t>(t);
+            if (st->m_len && ASRUtils::is_value_constant(st->m_len)) {
+                return st->m_len;
+            } else {
+                return ASRUtils::EXPR(ASR::make_StringLen_t(al, expr->base.loc,
+                    expr, ASRUtils::TYPE(ASR::make_Integer_t(al, expr->base.loc, 4)), nullptr));
+            }
+        }
+        return nullptr;
+    }
+
     ReplaceArrayConstant(Allocator& al_, Vec<ASR::stmt_t*>& pass_result_,
         bool& remove_original_statement_,
         std::map<ASR::expr_t*, ASR::expr_t*>& resultvar2value_,
         bool realloc_lhs_, bool allocate_target_) :
     al(al_), pass_result(pass_result_),
     remove_original_statement(remove_original_statement_),
-    current_scope(nullptr), result_var(nullptr), result_counter(0),
-    resultvar2value(resultvar2value_), realloc_lhs(realloc_lhs_),
-    allocate_target(allocate_target_) {}
+    current_scope(nullptr),
+    result_var(nullptr), result_counter(0),
+    resultvar2value(resultvar2value_),
+    realloc_lhs(realloc_lhs_), allocate_target(allocate_target_) {}
 
     ASR::expr_t* get_ImpliedDoLoop_size(ASR::ImpliedDoLoop_t* implied_doloop) {
         const Location& loc = implied_doloop->base.base.loc;
@@ -49,9 +160,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         int kind = ASRUtils::extract_kind_from_ttype_t(ASRUtils::expr_type(end));
         start = builder.i2i_t(start, ASRUtils::expr_type(end));
         if( d == nullptr ) {
-            implied_doloop_size = builder.Add(
-                builder.Sub(end, start),
-                make_ConstantWithKind(make_IntegerConstant_t, make_Integer_t, 1, kind, loc));
+            implied_doloop_size = ASRUtils::compute_length_from_start_end(al, start, end);
         } else {
             implied_doloop_size = builder.Add(builder.Div(
                 builder.Sub(end, start), d),
@@ -73,7 +182,8 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 const_elements += 1;
             }
         }
-        if( const_elements > 1 ) {
+        // Count scalar elements inside implied-do so mixed forms compute correctly per iteration.
+        if( const_elements > 0 ) {
             if( implied_doloop_size_ == nullptr ) {
                 implied_doloop_size_ = make_ConstantWithKind(make_IntegerConstant_t,
                     make_Integer_t, const_elements, kind, loc);
@@ -125,6 +235,24 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                     array_size = builder.Add(array_size,
                                     element_array_size);
                 }
+            } else if ( ASR::is_a<ASR::ArrayItem_t>(*element) ) {
+                ASR::ArrayItem_t* array_item = ASR::down_cast<ASR::ArrayItem_t>(element);
+                if ( ASR::is_a<ASR::Array_t>(*array_item->m_type) ) {
+                    if ( ASRUtils::is_fixed_size_array(array_item->m_type) ) {
+                        ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(array_item->m_type);
+                        constant_size += ASRUtils::get_fixed_size_of_array(array_type->m_dims, array_type->n_dims);
+                    } else {
+                        ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                        if( array_size == nullptr ) {
+                            array_size = element_array_size;
+                        } else {
+                            array_size = builder.Add(array_size,
+                                            element_array_size);
+                        }
+                    }
+                } else {
+                    constant_size += 1;
+                }
             } else if( ASR::is_a<ASR::Var_t>(*element) ) {
                 ASR::ttype_t* element_type = ASRUtils::type_get_past_allocatable(
                     ASRUtils::expr_type(element));
@@ -175,10 +303,82 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 if( array_size == nullptr ) {
                     array_size = array_section_size;
                 } else {
-                    builder.Add(array_section_size, array_size);
+                    array_size = builder.Add(array_section_size, array_size);
+                }
+            } else if( ASR::is_a<ASR::IntrinsicElementalFunction_t>(*element) &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) ) {
+                ASR::IntrinsicElementalFunction_t* intrinsic_element_t = ASR::down_cast<ASR::IntrinsicElementalFunction_t>(element);
+                if( ASRUtils::is_fixed_size_array(intrinsic_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(intrinsic_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( ASR::is_a<ASR::FunctionCall_t>(*element)  &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) ) {
+                ASR::FunctionCall_t* fc_element_t = ASR::down_cast<ASR::FunctionCall_t>(element);
+                if( ASRUtils::is_fixed_size_array(fc_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(fc_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( ASR::is_a<ASR::IntrinsicArrayFunction_t>(*element)  &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) ) {
+                ASR::IntrinsicArrayFunction_t* intrinsic_element_t = ASR::down_cast<ASR::IntrinsicArrayFunction_t>(element);
+                if( ASRUtils::is_fixed_size_array(intrinsic_element_t->m_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(intrinsic_element_t->m_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
+                }
+            } else if( (ASR::is_a<ASR::RealBinOp_t>(*element) || ASR::is_a<ASR::IntegerBinOp_t>(*element) ||
+                       ASR::is_a<ASR::RealUnaryMinus_t>(*element) || ASR::is_a<ASR::IntegerUnaryMinus_t>(*element)) &&
+                       ASRUtils::is_array(ASRUtils::expr_type(element)) ) {
+                ASR::ttype_t* arr_type = ASRUtils::expr_type(element);
+                if( ASRUtils::is_fixed_size_array(arr_type) ) {
+                    constant_size += ASRUtils::get_fixed_size_of_array(arr_type);
+                } else {
+                    ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                    if( array_size == nullptr ) {
+                        array_size = element_array_size;
+                    } else {
+                        array_size = builder.Add(array_size,
+                                        element_array_size);
+                    }
                 }
             } else {
-                constant_size += 1;
+                ASR::ttype_t* element_type = ASRUtils::type_get_past_allocatable(
+                    ASRUtils::type_get_past_pointer(ASRUtils::expr_type(element)));
+                if( ASRUtils::is_array(element_type) ) {
+                    if( ASRUtils::is_fixed_size_array(element_type) ) {
+                        constant_size += ASRUtils::get_fixed_size_of_array(element_type);
+                    } else {
+                        ASR::expr_t* element_array_size = ASRUtils::get_size(element, al, false);
+                        if( array_size == nullptr ) {
+                            array_size = element_array_size;
+                        } else {
+                            array_size = builder.Add(array_size, element_array_size);
+                        }
+                    }
+                } else {
+                    constant_size += 1;
+                }
             }
         }
         ASR::expr_t* constant_size_asr = nullptr;
@@ -221,6 +421,19 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         ASR::ttype_t* result_type_ = nullptr;
         bool is_allocatable = false;
         ASR::expr_t* array_constructor = get_ArrayConstructor_size(x, is_allocatable);
+
+        // Case: `keywords = [character(len=ii) :: value]`
+        // For runtime-dependent string lengths, defer evaluation and use allocatable results with per-element runtime allocation.
+        ASR::ttype_t* element_type = ASRUtils::extract_type(x->m_type);
+        ASR::expr_t* declared_len_expr = nullptr;
+        if (element_type && ASRUtils::is_character(*element_type)) {
+            ASR::String_t* string_type = ASR::down_cast<ASR::String_t>(element_type);
+            declared_len_expr = string_type->m_len;
+            bool len_is_constant = ASRUtils::is_value_constant(declared_len_expr);
+            if( !len_is_constant ) {
+                is_allocatable = true;
+            }
+        }
         Vec<ASR::dimension_t> dims;
         dims.reserve(al, 1);
         ASR::dimension_t dim;
@@ -240,21 +453,50 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 result_type_ = ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->m_type->base.loc,
                     ASRUtils::type_get_past_allocatable(
                         ASRUtils::duplicate_type_with_empty_dims(al, x->m_type))));
+                // Always defer string length for runtime-sized strings
+                if (element_type && ASRUtils::is_character(*element_type)) {
+                    ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(result_type_))->m_len = nullptr;
+                    ASR::down_cast<ASR::String_t>(ASRUtils::extract_type(result_type_))->m_len_kind = ASR::string_length_kindType::DeferredLength;
+                }
             } else {
                 result_type_ = ASRUtils::duplicate_type(al,
                     ASRUtils::type_get_past_allocatable(x->m_type), &dims);
+                // Keep fixed-size type as is; allocation will handle string length.
             }
         }
         result_var = PassUtils::create_var(result_counter, "_array_constructor_",
-                        loc, result_type_, al, current_scope);
+                        loc, result_type_, al, current_scope, x->m_args[0] ? x->m_args[0] : nullptr);
         result_counter += 1;
         *current_expr = result_var;
 
+        // Try to derive a safe element string length for allocation
+        ASR::expr_t* elem_len_expr_for_alloc = nullptr;
+        if (element_type && ASRUtils::is_character(*element_type)) {
+            // Default to declared length if available; override with a derived source length when safe.
+            elem_len_expr_for_alloc = declared_len_expr;
+            if (!declared_len_expr && x->n_args > 0) {
+                ASR::expr_t* sample_expr_any = get_first_scalar_expr(x->m_args[0]);
+                ASR::expr_t* derived_len = get_string_source_len(sample_expr_any);
+                if (derived_len) elem_len_expr_for_alloc = derived_len;
+            }
+        }
+
         Vec<ASR::alloc_arg_t> alloc_args;
         alloc_args.reserve(al, 1);
         ASR::alloc_arg_t arg;
-        arg.m_len_expr = nullptr;
+        // Prefer declared source length if available; else fall back to previously detected expr
+        arg.m_len_expr = elem_len_expr_for_alloc;
+        
+        // If we're allocating a deferred-length string but have no length expression,
+        // we cannot proceed. In this case, revert to non-deferred type.
+        if (is_allocatable && element_type && ASRUtils::is_character(*element_type) && 
+            !arg.m_len_expr) {
+            is_allocatable = false;
+            result_type_ = ASRUtils::duplicate_type(al,
+                ASRUtils::type_get_past_allocatable(x->m_type), &dims);
+        }
         arg.m_type = nullptr;
+        arg.m_sym_subclass = nullptr;
         arg.m_dims = dims.p;
         arg.n_dims = dims.size();
         if( is_allocatable ) {
@@ -288,10 +530,18 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
                 al, loc, alloc_args.p, alloc_args.size(), nullptr, nullptr, nullptr));
             pass_result.push_back(al, allocate_stmt);
         }
+        for (size_t i = 0; i < x->n_args; i++) {
+            if(ASR::is_a<ASR::ArrayItem_t>(*x->m_args[i]) || ASR::is_a<ASR::ArraySection_t>(*x->m_args[i])){
+                ASR::expr_t** temp = current_expr;
+                current_expr = &(x->m_args[i]);
+                self().replace_expr(x->m_args[i]);
+                current_expr = temp;
+            }
+        }
         LCOMPILERS_ASSERT(result_var != nullptr);
         Vec<ASR::stmt_t*>* result_vec = &pass_result;
-        PassUtils::ReplacerUtils::replace_ArrayConstructor(x, this,
-            remove_original_statement, result_vec);
+        PassUtils::ReplacerUtils::replace_ArrayConstructor_(al, x, result_var,
+            result_vec, current_scope);
         result_var = result_var_copy;
     }
 
@@ -323,7 +573,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
             is_allocatable = false;
         } else {
             if( is_allocatable ) {
-                result_type_ = ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->m_type->base.loc,
+                result_type_ = ASRUtils::TYPE(ASRUtils::make_Allocatable_t_util(al, x->m_type->base.loc,
                     ASRUtils::type_get_past_allocatable(
                         ASRUtils::duplicate_type_with_empty_dims(al, x->m_type))));
             } else {
@@ -341,6 +591,7 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
         ASR::alloc_arg_t arg;
         arg.m_len_expr = nullptr;
         arg.m_type = nullptr;
+        arg.m_sym_subclass = nullptr;
         arg.m_dims = dims.p;
         arg.n_dims = dims.size();
         if( is_allocatable ) {
@@ -382,9 +633,16 @@ class ReplaceArrayConstant: public ASR::BaseExprReplacer<ReplaceArrayConstant> {
     }
 
     void replace_ArrayPhysicalCast(ASR::ArrayPhysicalCast_t* x) {
+        [[maybe_unused]] bool is_arr_construct_arg = ASR::is_a<ASR::ArrayConstructor_t>(*x->m_arg);
         ASR::BaseExprReplacer<ReplaceArrayConstant>::replace_ArrayPhysicalCast(x);
-        // TODO: Allow for DescriptorArray to DescriptorArray physical cast for allocatables
-        // later on
+        if( x->m_old != ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg)) ) {
+            x->m_old = ASRUtils::extract_physical_type(ASRUtils::expr_type(x->m_arg));
+        }
+        if( (is_arr_construct_arg && ASRUtils::is_fixed_size_array(ASRUtils::expr_type(x->m_arg))) ){
+            *current_expr = x->m_arg;
+            return;
+        }
+
         if( (x->m_old == x->m_new &&
              x->m_old != ASR::array_physical_typeType::DescriptorArray) ||
             (x->m_old == x->m_new && x->m_old == ASR::array_physical_typeType::DescriptorArray &&
@@ -426,6 +684,8 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
         replacer(al_, pass_result, remove_original_statement,
             resultvar2value, realloc_lhs_, allocate_target),
         parent_body(nullptr) {
+            call_replacer_on_value = false;
+            replacer.call_replacer_on_value = false;
             pass_result.n = 0;
             pass_result.reserve(al, 0);
             print = false, file_write = false;
@@ -497,6 +757,8 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
             ASR::expr_t** current_expr_copy_9 = current_expr;
             current_expr = const_cast<ASR::expr_t**>(&(x.m_value));
             this->call_replacer();
+            current_expr = const_cast<ASR::expr_t**>(&(x.m_target));
+            this->call_replacer();
             current_expr = current_expr_copy_9;
             if( !remove_original_statement ) {
                 this->visit_expr(*x.m_value);
@@ -550,15 +812,20 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                     args.reserve(al, 1);
                     args.push_back(al, x->m_values[i]);
                     ASR::expr_t* fmt_val = ASRUtils::EXPR(ASR::make_StringFormat_t(al,x->base.base.loc, nullptr,
-                        args.p, 1,ASR::string_format_kindType::FormatFortran, 
-                        ASRUtils::TYPE(ASR::make_Character_t(al,x->base.base.loc,-1,-1,nullptr, ASR::string_physical_typeType::PointerString)), nullptr));
+                        args.p, 1, ASR::string_format_kindType::FormatFortran,
+                        ASRUtils::TYPE(ASR::make_Allocatable_t(al, x->base.base.loc, 
+                            ASRUtils::TYPE(ASR::make_String_t(al,x->base.base.loc, 1,
+                                nullptr,
+                                ASR::string_length_kindType::DeferredLength,
+                                ASR::string_physical_typeType::DescriptorString)))),
+                        nullptr));
                     print_values.push_back(al, fmt_val);
                     if ( print ) {
                         stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, x->m_values[i]->base.loc, print_values.p, print_values.size()));
                     } else {
                         // this will be file_write
                         LCOMPILERS_ASSERT(file_write);
-                        stmt = ASRUtils::STMT(ASR::make_FileWrite_t(al, x->m_values[i]->base.loc, 0, m_unit, nullptr, nullptr, nullptr, print_values.p, print_values.size(), nullptr, nullptr, nullptr));
+                        stmt = ASRUtils::STMT(ASR::make_FileWrite_t(al, x->m_values[i]->base.loc, 0, m_unit, nullptr, nullptr, nullptr, print_values.p, print_values.size(), nullptr, nullptr, nullptr, true, nullptr));
                     }
                     do_loop_body.push_back(al, stmt);
                 }
@@ -578,7 +845,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
                 integer :: i
                 print *, [(i, i=1, 10)]
             */
-            if(ASR::is_a<ASR::Character_t>(*ASRUtils::expr_type(x.m_text))){
+            if(ASR::is_a<ASR::String_t>(*ASRUtils::extract_type(ASRUtils::expr_type(x.m_text)))){
                 ASR::Print_t* print_stmt = const_cast<ASR::Print_t*>(&x);
                 ASR::expr_t** current_expr_copy_9 = current_expr;
                 current_expr = const_cast<ASR::expr_t**>(&(print_stmt->m_text));
@@ -720,7 +987,7 @@ class ArrayConstantVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayC
 void pass_replace_implied_do_loops(Allocator &al,
     ASR::TranslationUnit_t &unit,
     const LCompilers::PassOptions& pass_options) {
-    ArrayConstantVisitor v(al, pass_options.realloc_lhs);
+    ArrayConstantVisitor v(al, pass_options.realloc_lhs_arrays);
     v.visit_TranslationUnit(unit);
     PassUtils::UpdateDependenciesVisitor u(al);
     u.visit_TranslationUnit(unit);
