@@ -20,7 +20,6 @@ inline std::string get_intrinsic_name(int64_t x) {
     switch (x) {
         INTRINSIC_NAME_CASE(ObjectType)
         INTRINSIC_NAME_CASE(Kind)
-        INTRINSIC_NAME_CASE(Rank)
         INTRINSIC_NAME_CASE(Sin)
         INTRINSIC_NAME_CASE(Cos)
         INTRINSIC_NAME_CASE(Tan)
@@ -75,8 +74,6 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(BesselY1)
         INTRINSIC_NAME_CASE(BesselYN)
         INTRINSIC_NAME_CASE(SameTypeAs)
-        INTRINSIC_NAME_CASE(Mvbits)
-        INTRINSIC_NAME_CASE(MoveAlloc)
         INTRINSIC_NAME_CASE(Merge)
         INTRINSIC_NAME_CASE(Mergebits)
         INTRINSIC_NAME_CASE(Shiftr)
@@ -98,8 +95,11 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(SetExponent)
         INTRINSIC_NAME_CASE(Not)
         INTRINSIC_NAME_CASE(Iand)
+        INTRINSIC_NAME_CASE(And)
         INTRINSIC_NAME_CASE(Ior)
+        INTRINSIC_NAME_CASE(Or)
         INTRINSIC_NAME_CASE(Ieor)
+        INTRINSIC_NAME_CASE(Xor)
         INTRINSIC_NAME_CASE(Ibclr)
         INTRINSIC_NAME_CASE(Ibset)
         INTRINSIC_NAME_CASE(Btest)
@@ -114,19 +114,22 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(SubstrIndex)
         INTRINSIC_NAME_CASE(Range)
         INTRINSIC_NAME_CASE(Radix)
-        INTRINSIC_NAME_CASE(IsContiguous)
         INTRINSIC_NAME_CASE(StorageSize)
         INTRINSIC_NAME_CASE(Hypot)
         INTRINSIC_NAME_CASE(SelectedIntKind)
         INTRINSIC_NAME_CASE(SelectedRealKind)
         INTRINSIC_NAME_CASE(SelectedCharKind)
+        INTRINSIC_NAME_CASE(SelectedLogicalKind)
+        INTRINSIC_NAME_CASE(Present)
         INTRINSIC_NAME_CASE(Adjustl)
         INTRINSIC_NAME_CASE(Adjustr)
+        INTRINSIC_NAME_CASE(StringConcat)
         INTRINSIC_NAME_CASE(StringLenTrim)
         INTRINSIC_NAME_CASE(StringTrim)
         INTRINSIC_NAME_CASE(Ichar)
         INTRINSIC_NAME_CASE(Char)
         INTRINSIC_NAME_CASE(Achar)
+        INTRINSIC_NAME_CASE(Iachar)
         INTRINSIC_NAME_CASE(MinExponent)
         INTRINSIC_NAME_CASE(MaxExponent)
         INTRINSIC_NAME_CASE(Ishftc)
@@ -144,10 +147,12 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(Sign)
         INTRINSIC_NAME_CASE(SignFromValue)
         INTRINSIC_NAME_CASE(Nint)
+        INTRINSIC_NAME_CASE(Idnint)
         INTRINSIC_NAME_CASE(Aint)
         INTRINSIC_NAME_CASE(Popcnt)
         INTRINSIC_NAME_CASE(Poppar)
         INTRINSIC_NAME_CASE(Real)
+        INTRINSIC_NAME_CASE(Cmplx)
         INTRINSIC_NAME_CASE(Dim)
         INTRINSIC_NAME_CASE(Anint)
         INTRINSIC_NAME_CASE(Sqrt)
@@ -166,6 +171,7 @@ inline std::string get_intrinsic_name(int64_t x) {
         INTRINSIC_NAME_CASE(NewLine)
         INTRINSIC_NAME_CASE(Conjg)
         INTRINSIC_NAME_CASE(Huge)
+        INTRINSIC_NAME_CASE(Loc)
         INTRINSIC_NAME_CASE(Dprod)
         INTRINSIC_NAME_CASE(SymbolicSymbol)
         INTRINSIC_NAME_CASE(SymbolicAdd)
@@ -199,9 +205,12 @@ inline std::string get_intrinsic_name(int64_t x) {
 
 namespace IntrinsicElementalFunctionRegistry {
 
-    static const std::map<int64_t,
+    inline const std::map<int64_t,
         std::tuple<impl_function,
-                   verify_function>>& intrinsic_function_by_id_db = {
+                   verify_function>>& get_intrinsic_function_by_id_db() {
+        static const std::map<int64_t,
+            std::tuple<impl_function,
+                       verify_function>> intrinsic_function_by_id_db = {
         {static_cast<int64_t>(IntrinsicElementalFunctions::ObjectType),
             {nullptr, &ObjectType::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Gamma),
@@ -308,10 +317,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {nullptr, &SameTypeAs::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Merge),
             {&Merge::instantiate_Merge, &Merge::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Mvbits),
-            {&Mvbits::instantiate_Mvbits, &Mvbits::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::MoveAlloc),
-            {&MoveAlloc::instantiate_MoveAlloc, &MoveAlloc::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Mergebits),
             {&Mergebits::instantiate_Mergebits, &Mergebits::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftr),
@@ -320,6 +325,8 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Adjustl::instantiate_Adjustl, &Adjustl::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Adjustr),
             {&Adjustr::instantiate_Adjustr, &Adjustr::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::StringConcat),
+            {&StringConcat::instantiate_StringConcat, &StringConcat::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StringLenTrim),
             {&StringLenTrim::instantiate_StringLenTrim, &StringLenTrim::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StringTrim),
@@ -330,6 +337,8 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Char::instantiate_Char, &Char::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Achar),
             {&Achar::instantiate_Achar, &Achar::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Iachar),
+            {&Iachar::instantiate_Iachar, &Iachar::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Rshift),
             {&Rshift::instantiate_Rshift, &Rshift::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftl),
@@ -366,10 +375,16 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Not::instantiate_Not, &Not::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Iand),
             {&Iand::instantiate_Iand, &Iand::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::And),
+            {&And::instantiate_And, &And::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Ior),
             {&Ior::instantiate_Ior, &Ior::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Or),
+            {&Or::instantiate_Or, &Or::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Ieor),
             {&Ieor::instantiate_Ieor, &Ieor::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Xor),
+            {&Xor::instantiate_Xor, &Xor::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Ibclr),
             {&Ibclr::instantiate_Ibclr, &Ibclr::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Btest),
@@ -386,8 +401,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Hypot::instantiate_Hypot, &Hypot::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Kind),
             {nullptr, &Kind::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rank),
-            {nullptr, &Rank::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Digits),
             {&Digits::instantiate_Digits, &Digits::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Rrspacing),
@@ -436,8 +449,6 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Sign::instantiate_Sign, &Sign::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
             {nullptr, &Radix::verify_args}},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            {nullptr, &IsContiguous::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
             {nullptr, &StorageSize::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
@@ -454,8 +465,12 @@ namespace IntrinsicElementalFunctionRegistry {
             {&Poppar::instantiate_Poppar, &Poppar::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Real),
             {&Real::instantiate_Real, &Real::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Cmplx),
+            {&Cmplx::instantiate_Cmplx, &Cmplx::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Nint),
             {&Nint::instantiate_Nint, &Nint::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Idnint),
+            {&Idnint::instantiate_Idnint, &Idnint::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Anint),
             {&Anint::instantiate_Anint, &Anint::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Dim),
@@ -494,12 +509,18 @@ namespace IntrinsicElementalFunctionRegistry {
             {nullptr, &UnaryIntrinsicFunction::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Huge),
             {nullptr, &UnaryIntrinsicFunction::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Loc),
+            {nullptr, &UnaryIntrinsicFunction::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedIntKind),
             {&SelectedIntKind::instantiate_SelectedIntKind, &SelectedIntKind::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Present),
+            {&Present::instantiate_Present, &Present::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedRealKind),
             {&SelectedRealKind::instantiate_SelectedRealKind, &SelectedRealKind::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedCharKind),
             {&SelectedCharKind::instantiate_SelectedCharKind, &SelectedCharKind::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedLogicalKind),
+            {&SelectedLogicalKind::instantiate_SelectedLogicalKind, &SelectedLogicalKind::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSymbol),
             {nullptr, &SymbolicSymbol::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicAdd),
@@ -550,359 +571,17 @@ namespace IntrinsicElementalFunctionRegistry {
             {&CommandArgumentCount::instantiate_CommandArgumentCount, &CommandArgumentCount::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Int),
             {&Int::instantiate_Int, &Int::verify_args}},
-    };
-
-    static const std::map<int64_t, std::string>& intrinsic_function_id_to_name = {
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ObjectType),
-            "type"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Gamma),
-            "gamma"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Log),
-            "log"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Log10),
-            "log10"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Logical),
-            "logical"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::LogGamma),
-            "log_gamma"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Erf),
-            "erf"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Erfc),
-            "erfc"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ErfcScaled),
-            "erfc_scaled"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Trunc),
-            "trunc"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Fix),
-            "fix"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sin),
-            "sin"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Cos),
-            "cos"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Tan),
-            "tan"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Asin),
-            "asin"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Acos),
-            "acos"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Atan),
-            "atan"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sinh),
-            "sinh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Cosh),
-            "cosh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Tanh),
-            "tanh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Atan2),
-            "atan2"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Asinh),
-            "asinh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Acosh),
-            "acosh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Atanh),
-            "atanh"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Abs),
-            "abs"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Aimag),
-            "aimag"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Dreal),
-            "dreal"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Exp),
-            "exp"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Exp2),
-            "exp2"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::FMA),
-            "fma"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::FlipSign),
-            "flipsign"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::FloorDiv),
-            "floordiv"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Mod),
-            "mod"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Trailz),
-            "trailz"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Isnan),
-            "isnan"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Nearest),
-            "nearest"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::CompilerVersion),
-            "compiler_version"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::CommandArgumentCount),
-            "command_argument_count"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Spacing),
-            "spacing"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Modulo),
-            "modulo"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Asin),
-            "asind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Acos),
-            "acosd"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Atan),
-            "atand"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sind),
-            "sind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Cosd),
-            "cosd"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Tand),
-            "tand"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::OutOfRange),
-            "out_of_range"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJ0),
-            "bessel_j0"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJ1),
-            "bessel_j1"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY0),
-            "bessel_y0"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY1),
-            "bessel_y1"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJN),
-            "bessel_jn"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselYN),
-            "bessel_yn"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SameTypeAs),
-            "same_type_as"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Merge),
-            "merge"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Mvbits),
-            "mvbits"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::MoveAlloc),
-            "move_alloc"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Mergebits),
-            "mergebits"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftr),
-            "shiftr"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rshift),
-            "rshift"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Adjustl),
-            "adjustl"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Adjustr),
-            "adjustr"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::StringLenTrim),
-            "len_trim"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::StringTrim),
-            "trim"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ichar),
-            "ichar"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Char),
-            "char"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Achar),
-            "achar"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftl),
-            "shiftl"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Dshiftl),
-           "dshiftl"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Dshiftr),
-            "dshiftr"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ishft),
-            "ishft"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Bgt),
-            "bgt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Blt),
-            "blt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Bge),
-            "bge"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ble),
-            "ble"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Lgt),
-            "lgt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Llt),
-            "llt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Lge),
-            "lge"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Lle),
-            "lle"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Exponent),
-            "exponent"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Fraction),
-            "fraction"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SetExponent),
-            "set_exponent"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Not),
-            "not"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Iand),
-            "iand"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ior),
-            "ior"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ieor),
-            "ieor"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ibclr),
-            "ibclr"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ibset),
-            "ibset"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Btest),
-            "btest"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ibits),
-            "ibits"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Leadz),
-            "leadz"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ToLowerCase),
-            "_lfortran_tolowercase"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Hypot),
-            "hypot"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedIntKind),
-            "selected_int_kind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedRealKind),
-            "selected_real_kind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedCharKind),
-            "selected_char_kind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Kind),
-            "kind"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rank),
-            "rank"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Digits),
-            "Digits"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Rrspacing),
-            "rrspacing"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Repeat),
-            "Repeat"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::StringContainsSet),
-            "Verify"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::StringFindSet),
-            "Scan"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SubstrIndex),
-            "Index"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::MinExponent),
-            "minexponent"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::MaxExponent),
-            "maxexponent"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Expm1),
-            "expm1"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ListIndex),
-            "list.index"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ListReverse),
-            "list.reverse"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ListPop),
-            "list.pop"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::ListReserve),
-            "list.reserve"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::DictKeys),
-            "dict.keys"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::DictValues),
-            "dict.values"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SetAdd),
-            "set.add"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SetRemove),
-            "set.remove"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Max),
-            "max"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Min),
-            "min"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ishftc),
-            "ishftc"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Radix),
-            "radix"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::IsContiguous),
-            "is_contiguous"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::StorageSize),
-            "storage_size"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Scale),
-            "scale"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Dprod),
-            "dprod"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Range),
-            "range"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sign),
-            "sign"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Aint),
-            "aint"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Popcnt),
-            "popcnt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Poppar),
-            "poppar"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Real),
-            "real"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Nint),
-            "nint"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Anint),
-            "anint"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Dim),
-            "dim"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Floor),
-            "floor"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ceiling),
-            "ceiling"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Maskr),
-            "Maskr"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Maskl),
-            "maskl"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sqrt),
-            "sqrt"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Sngl),
-            "sngl"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Idint),
-            "idint"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Ifix),
-            "ifix"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Conjg),
-            "conjg"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SignFromValue),
-            "signfromvalue"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Epsilon),
-            "epsilon"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Precision),
-            "precision"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Tiny),
-            "tiny"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::BitSize),
-            "bit_size"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::NewLine),
-            "new_line"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Huge),
-            "huge"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSymbol),
-            "Symbol"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicAdd),
-            "SymbolicAdd"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSub),
-            "SymbolicSub"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicMul),
-            "SymbolicMul"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicDiv),
-            "SymbolicDiv"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicPow),
-            "SymbolicPow"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicPi),
-            "pi"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicE),
-            "E"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicInteger),
-            "SymbolicInteger"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicDiff),
-            "SymbolicDiff"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicExpand),
-            "SymbolicExpand"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSin),
-            "SymbolicSin"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicCos),
-            "SymbolicCos"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicLog),
-            "SymbolicLog"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicExp),
-            "SymbolicExp"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicAbs),
-            "SymbolicAbs"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicHasSymbolQ),
-            "SymbolicHasSymbolQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicAddQ),
-            "SymbolicAddQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicMulQ),
-            "SymbolicMulQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicPowQ),
-            "SymbolicPowQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicLogQ),
-            "SymbolicLogQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSinQ),
-            "SymbolicSinQ"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicGetArgument),
-            "SymbolicGetArgument"},
-        {static_cast<int64_t>(IntrinsicElementalFunctions::Int),
-            "int"},
-    };
+        };
+        return intrinsic_function_by_id_db;
+    }
 
 
-    static const std::map<std::string,
+    inline const std::map<std::string,
         std::tuple<create_intrinsic_function,
-                    eval_intrinsic_function>>& intrinsic_function_by_name_db = {
+                    eval_intrinsic_function>>& get_intrinsic_function_by_name_db() {
+        static const std::map<std::string,
+            std::tuple<create_intrinsic_function,
+                        eval_intrinsic_function>> intrinsic_function_by_name_db = {
                 {"type", {&ObjectType::create_ObjectType, &ObjectType::eval_ObjectType}},
                 {"gamma", {&Gamma::create_Gamma, &Gamma::eval_Gamma}},
                 {"log", {&Log::create_Log, &Log::eval_Log}},
@@ -949,15 +628,13 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"trailz", {&Trailz::create_Trailz, &Trailz::eval_Trailz}},
                 {"isnan", {&Isnan::create_Isnan, &Isnan::eval_Isnan}},
                 {"nearest", {&Nearest::create_Nearest, &Nearest::eval_Nearest}},
-                {"compiler_version", {&CompilerVersion::create_CompilerVersion, &CompilerVersion::eval_CompilerVersion}},
+                {"_lfortran_compiler_version", {&CompilerVersion::create_CompilerVersion, &CompilerVersion::eval_CompilerVersion}},
                 {"command_argument_count", {&CommandArgumentCount::create_CommandArgumentCount, nullptr}},
                 {"spacing", {&Spacing::create_Spacing, &Spacing::eval_Spacing}},
                 {"modulo", {&Modulo::create_Modulo, &Modulo::eval_Modulo}},
                 {"bessel_jn", {&BesselJN::create_BesselJN, &BesselJN::eval_BesselJN}},
                 {"bessel_yn", {&BesselYN::create_BesselYN, &BesselYN::eval_BesselYN}},
                 {"merge", {&Merge::create_Merge, &Merge::eval_Merge}},
-                {"mvbits", {&Mvbits::create_Mvbits, &Mvbits::eval_Mvbits}},
-                {"move_alloc", {&MoveAlloc::create_MoveAlloc, &MoveAlloc::eval_MoveAlloc}},
                 {"merge_bits", {&Mergebits::create_Mergebits, &Mergebits::eval_Mergebits}},
                 {"shiftr", {&Shiftr::create_Shiftr, &Shiftr::eval_Shiftr}},
                 {"rshift", {&Rshift::create_Rshift, &Rshift::eval_Rshift}},
@@ -980,8 +657,11 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"set_exponent", {&SetExponent::create_SetExponent, &SetExponent::eval_SetExponent}},
                 {"not", {&Not::create_Not, &Not::eval_Not}},
                 {"iand", {&Iand::create_Iand, &Iand::eval_Iand}},
+                {"and", {&And::create_And, &And::eval_And}},
                 {"ior", {&Ior::create_Ior, &Ior::eval_Ior}},
+                {"or", {&Or::create_Or, &Or::eval_Or}},
                 {"ieor", {&Ieor::create_Ieor, &Ieor::eval_Ieor}},
+                {"xor", {&Xor::create_Xor, &Xor::eval_Xor}},
                 {"ibclr", {&Ibclr::create_Ibclr, &Ibclr::eval_Ibclr}},
                 {"ibset", {&Ibset::create_Ibset, &Ibset::eval_Ibset}},
                 {"btest", {&Btest::create_Btest, &Btest::eval_Btest}},
@@ -990,10 +670,11 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"_lfortran_tolowercase", {&ToLowerCase::create_ToLowerCase, &ToLowerCase::eval_ToLowerCase}},
                 {"hypot", {&Hypot::create_Hypot, &Hypot::eval_Hypot}},
                 {"selected_int_kind", {&SelectedIntKind::create_SelectedIntKind, &SelectedIntKind::eval_SelectedIntKind}},
+                {"present", {&Present::create_Present, &Present::eval_Present}},
                 {"selected_real_kind", {&SelectedRealKind::create_SelectedRealKind, &SelectedRealKind::eval_SelectedRealKind}},
                 {"selected_char_kind", {&SelectedCharKind::create_SelectedCharKind, &SelectedCharKind::eval_SelectedCharKind}},
+                {"selected_logical_kind", {&SelectedLogicalKind::create_SelectedLogicalKind, &SelectedLogicalKind::eval_SelectedLogicalKind}},
                 {"kind", {&Kind::create_Kind, &Kind::eval_Kind}},
-                {"rank", {&Rank::create_Rank, &Rank::eval_Rank}},
                 {"digits", {&Digits::create_Digits, &Digits::eval_Digits}},
                 {"rrspacing", {&Rrspacing::create_Rrspacing, &Rrspacing::eval_Rrspacing}},
                 {"repeat", {&Repeat::create_Repeat, &Repeat::eval_Repeat}},
@@ -1013,17 +694,18 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"max0", {&Max::create_Max, &Max::eval_Max}},
                 {"adjustl", {&Adjustl::create_Adjustl, &Adjustl::eval_Adjustl}},
                 {"adjustr", {&Adjustr::create_Adjustr, &Adjustr::eval_Adjustr}},
+                {"stringconcat", {&StringConcat::create_StringConcat, &StringConcat::eval_StringConcat}},
                 {"len_trim", {&StringLenTrim::create_StringLenTrim, &StringLenTrim::eval_StringLenTrim}},
                 {"trim", {&StringTrim::create_StringTrim, &StringTrim::eval_StringTrim}},
                 {"ichar", {&Ichar::create_Ichar, &Ichar::eval_Ichar}},
                 {"char", {&Char::create_Char, &Char::eval_Char}},
                 {"achar", {&Achar::create_Achar, &Achar::eval_Achar}},
+                {"iachar", {&Iachar::create_Iachar, &Iachar::eval_Iachar}},
                 {"min0", {&Min::create_Min, &Min::eval_Min}},
                 {"max", {&Max::create_Max, &Max::eval_Max}},
                 {"min", {&Min::create_Min, &Min::eval_Min}},
                 {"ishftc", {&Ishftc::create_Ishftc, &Ishftc::eval_Ishftc}},
                 {"radix", {&Radix::create_Radix, &Radix::eval_Radix}},
-                {"is_contiguous", {&IsContiguous::create_IsContiguous, &IsContiguous::eval_IsContiguous}},
                 {"out_of_range", {&OutOfRange::create_OutOfRange, &OutOfRange::eval_OutOfRange}},
                 {"storage_size", {&StorageSize::create_StorageSize, &StorageSize::eval_StorageSize}},
                 {"scale", {&Scale::create_Scale, &Scale::eval_Scale}},
@@ -1034,7 +716,9 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"popcnt", {&Popcnt::create_Popcnt, &Popcnt::eval_Popcnt}},
                 {"poppar", {&Poppar::create_Poppar, &Poppar::eval_Poppar}},
                 {"real", {&Real::create_Real, &Real::eval_Real}},
+                {"cmplx", {&Cmplx::create_Cmplx, &Cmplx::eval_Cmplx}},
                 {"nint", {&Nint::create_Nint, &Nint::eval_Nint}},
+                {"idnint", {&Idnint::create_Idnint, &Idnint::eval_Idnint}},
                 {"anint", {&Anint::create_Anint, &Anint::eval_Anint}},
                 {"dim", {&Dim::create_Dim, &Dim::eval_Dim}},
                 {"floor", {&Floor::create_Floor, &Floor::eval_Floor}},
@@ -1052,6 +736,7 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"new_line", {&NewLine::create_NewLine, &NewLine::eval_NewLine}},
                 {"conjg", {&Conjg::create_Conjg, &Conjg::eval_Conjg}},
                 {"huge", {&Huge::create_Huge, &Huge::eval_Huge}},
+                {"loc", {&Loc::create_Loc, &Loc::eval_Loc}},
                 {"Symbol", {&SymbolicSymbol::create_SymbolicSymbol, &SymbolicSymbol::eval_SymbolicSymbol}},
                 {"SymbolicAdd", {&SymbolicAdd::create_SymbolicAdd, &SymbolicAdd::eval_SymbolicAdd}},
                 {"SymbolicSub", {&SymbolicSub::create_SymbolicSub, &SymbolicSub::eval_SymbolicSub}},
@@ -1076,37 +761,36 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"SinQ", {&SymbolicSinQ::create_SymbolicSinQ, &SymbolicSinQ::eval_SymbolicSinQ}},
                 {"GetArgument", {&SymbolicGetArgument::create_SymbolicGetArgument, &SymbolicGetArgument::eval_SymbolicGetArgument}},
                 {"int", {&Int::create_Int, &Int::eval_Int}},
-    };
+        };
+        return intrinsic_function_by_name_db;
+    }
 
     static inline bool is_intrinsic_function(const std::string& name) {
-        return intrinsic_function_by_name_db.find(name) != intrinsic_function_by_name_db.end();
+        return get_intrinsic_function_by_name_db().find(name) != get_intrinsic_function_by_name_db().end();
     }
 
     static inline bool is_intrinsic_function(int64_t id) {
-        return intrinsic_function_by_id_db.find(id) != intrinsic_function_by_id_db.end();
+        return get_intrinsic_function_by_id_db().find(id) != get_intrinsic_function_by_id_db().end();
     }
 
     static inline create_intrinsic_function get_create_function(const std::string& name) {
-        return std::get<0>(intrinsic_function_by_name_db.at(name));
+        return std::get<0>(get_intrinsic_function_by_name_db().at(name));
     }
 
     static inline verify_function get_verify_function(int64_t id) {
-        return std::get<1>(intrinsic_function_by_id_db.at(id));
+        return std::get<1>(get_intrinsic_function_by_id_db().at(id));
     }
 
     static inline impl_function get_instantiate_function(int64_t id) {
-        if( intrinsic_function_by_id_db.find(id) == intrinsic_function_by_id_db.end() ) {
+        if( get_intrinsic_function_by_id_db().find(id) == get_intrinsic_function_by_id_db().end() ) {
             return nullptr;
         }
-        return std::get<0>(intrinsic_function_by_id_db.at(id));
+        return std::get<0>(get_intrinsic_function_by_id_db().at(id));
     }
 
     static inline std::string get_intrinsic_function_name(int64_t id) {
-        if( intrinsic_function_id_to_name.find(id) == intrinsic_function_id_to_name.end() ) {
-            throw LCompilersException("IntrinsicFunction with ID " + std::to_string(id) +
-                                      " has no name registered for it");
-        }
-        return intrinsic_function_id_to_name.at(id);
+        // Use switch statement instead of lazy map for zero runtime overhead
+        return get_intrinsic_name(id);
     }
 
 } // namespace IntrinsicElementalFunctionRegistry
@@ -1169,19 +853,23 @@ namespace IsIostatEor {
 
 namespace IntrinsicImpureFunctionRegistry {
 
-    static const std::map<std::string, std::tuple<create_intrinsic_function,
-            eval_intrinsic_function>>& function_by_name_db = {
-        {"is_iostat_end", {&IsIostatEnd::create_IsIostatEnd, nullptr}},
-        {"is_iostat_eor", {&IsIostatEor::create_IsIostatEor, nullptr}},
-        {"allocated", {&Allocated::create_Allocated, nullptr}},
-    };
+    inline const std::map<std::string, std::tuple<create_intrinsic_function,
+            eval_intrinsic_function>>& get_function_by_name_db() {
+        static const std::map<std::string, std::tuple<create_intrinsic_function,
+                eval_intrinsic_function>> function_by_name_db = {
+            {"is_iostat_end", {&IsIostatEnd::create_IsIostatEnd, nullptr}},
+            {"is_iostat_eor", {&IsIostatEor::create_IsIostatEor, nullptr}},
+            {"allocated", {&Allocated::create_Allocated, nullptr}},
+        };
+        return function_by_name_db;
+    }
 
     static inline bool is_intrinsic_function(const std::string& name) {
-        return function_by_name_db.find(name) != function_by_name_db.end();
+        return get_function_by_name_db().find(name) != get_function_by_name_db().end();
     }
 
     static inline create_intrinsic_function get_create_function(const std::string& name) {
-        return  std::get<0>(function_by_name_db.at(name));
+        return  std::get<0>(get_function_by_name_db().at(name));
     }
 
 } // namespace IntrinsicImpureFunctionRegistry
