@@ -58,8 +58,12 @@ public:
         Vec<ASR::expr_t*> idx_vars;
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
-        ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_Character_t(
-            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+        ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
+            al, loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len_2));
         ASR::stmt_t* empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, loc, empty_space));
@@ -87,7 +91,7 @@ public:
                     format_args.push_back(al, string_format);
                     print_stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
                         format_args.p, format_args.size()));
-                } else if (ASR::is_a<ASR::Character_t>(*ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_array(ASRUtils::expr_type(print_args[0]))))) {
+                } else if (ASR::is_a<ASR::String_t>(*ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_array(ASRUtils::expr_type(print_args[0]))))) {
                     print_stmt = ASRUtils::STMT(ASRUtils::make_print_t_util(al, loc,
                         print_args.p, print_args.size()));
                 } else {
@@ -104,24 +108,46 @@ public:
         return doloop;
     }
 
-    void print_fixed_sized_array(ASR::expr_t *arr_expr,std::vector<ASR::expr_t*>& print_body, const Location &loc) {
-        ASR::dimension_t* m_dims;
-        int n_dims = ASRUtils::extract_dimensions_from_ttype(ASRUtils::expr_type(arr_expr), m_dims);
-        int m_dim_length = ASR::down_cast<ASR::IntegerConstant_t>(m_dims->m_length)->m_n;
-        Vec<ASR::expr_t*> idx_vars;
-        PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
+    void print_fixed_sized_array_helper(ASR::expr_t *arr_expr, std::vector<ASR::expr_t*> &print_body, 
+                                        const Location &loc, std::vector<ASR::expr_t*> &current_indices, 
+                                        int dim_index, int n_dims, ASR::dimension_t* m_dims, 
+                                        Allocator &al, SymbolTable *current_scope) {
         ASR::ttype_t *int32_type = ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4));
         ASR::expr_t* one = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 1, int32_type));
-        for (int n = 0; n < n_dims; n++) {
-            ASR::expr_t* idx_var = idx_vars[n];
-            idx_var = one;
-            for (int m = 0; m < m_dim_length; m++) {
-                ASR::expr_t* ref = PassUtils::create_array_ref(arr_expr, idx_var, al, current_scope);
+
+        int m_dim_length = ASR::down_cast<ASR::IntegerConstant_t>(m_dims[dim_index].m_length)->m_n;
+
+        for (int i = 0; i < m_dim_length; i++) {
+            if (dim_index == 0) {
+                Vec<ASR::expr_t*> indices;
+                indices.reserve(al, n_dims);
+                for (int j = 0; j < n_dims; j++) {
+                    indices.push_back(al, current_indices[j]);
+                }
+
+                ASR::expr_t* ref = PassUtils::create_array_ref(arr_expr, indices, al, current_scope);
                 print_body.push_back(ref);
-                idx_var = ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, loc, idx_var,
-                    ASR::binopType::Add, one, int32_type, nullptr));
+            } else {
+                print_fixed_sized_array_helper(arr_expr, print_body, loc, current_indices, 
+                                                dim_index - 1, n_dims, m_dims, al, current_scope);
             }
+
+            current_indices[dim_index] = ASRUtils::EXPR(ASR::make_IntegerBinOp_t(
+                al, loc, current_indices[dim_index], ASR::binopType::Add, one, int32_type, nullptr));
         }
+        current_indices[dim_index] = PassUtils::get_bound(arr_expr, dim_index + 1, "lbound", al);
+    }
+
+    void print_fixed_sized_array(ASR::expr_t *arr_expr, std::vector<ASR::expr_t*> &print_body, const Location &loc) {
+        ASR::dimension_t* m_dims;
+        int n_dims = ASRUtils::extract_dimensions_from_ttype(ASRUtils::expr_type(arr_expr), m_dims);
+
+        std::vector<ASR::expr_t*> current_indices(n_dims);
+        for (int i = 0; i < n_dims; i++) {
+            current_indices[i] = PassUtils::get_bound(arr_expr, i + 1, "lbound", al);
+        }
+
+        print_fixed_sized_array_helper(arr_expr, print_body, loc, current_indices, n_dims - 1, n_dims, m_dims, al, current_scope);
     }
 
     ASR::stmt_t* create_formatstmt(std::vector<ASR::expr_t*> &print_body, ASR::StringFormat_t* format, const Location &loc, ASR::stmtType _type,
@@ -143,20 +169,24 @@ public:
                 print_args.p, print_args.size()));
         } else if (_type == ASR::stmtType::FileWrite) {
             statement = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc, 0, unit,
-                nullptr, nullptr, nullptr, print_args.p, print_args.size(), separator, end, overloaded));
+                nullptr, nullptr, nullptr, print_args.p, print_args.size(), separator, end, overloaded, true, nullptr));
         }
         print_body.clear();
         return statement;
     }
 
     void visit_Print(const ASR::Print_t& x) {
-        LCOMPILERS_ASSERT(ASR::is_a<ASR::Character_t>(*ASRUtils::expr_type(x.m_text)));
+        LCOMPILERS_ASSERT(ASR::is_a<ASR::String_t>(*ASRUtils::extract_type(ASRUtils::expr_type(x.m_text))));
         if (ASR::is_a<ASR::StringFormat_t>(*x.m_text)) {
             std::vector<ASR::expr_t*> print_body;
             ASR::stmt_t* empty_print_endl;
             ASR::stmt_t* print_stmt;
-            ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_Character_t(
-            al, x.base.base.loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+            ASR::ttype_t *str_type_len_2 = ASRUtils::TYPE(ASR::make_String_t(
+            al, x.base.base.loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, x.base.base.loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, x.base.base.loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
             ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, x.base.base.loc, s2c(al, ""), str_type_len_2));
             empty_print_endl = ASRUtils::STMT(ASR::make_Print_t(al, x.base.base.loc, empty_space));
@@ -193,12 +223,16 @@ public:
         Vec<ASR::expr_t*> idx_vars;
         PassUtils::create_idx_vars(idx_vars, n_dims, loc, al, current_scope);
         ASR::stmt_t* doloop = nullptr;
-        ASR::ttype_t *str_type_len = ASRUtils::TYPE(ASR::make_Character_t(
-            al, loc, 1, 0, nullptr, ASR::string_physical_typeType::PointerString));
+        ASR::ttype_t *str_type_len = ASRUtils::TYPE(ASR::make_String_t(
+            al, loc, 1, 
+            ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, loc, 0,
+                ASRUtils::TYPE(ASR::make_Integer_t(al, loc, 4)))),
+            ASR::string_length_kindType::ExpressionLength,
+            ASR::string_physical_typeType::DescriptorString));
         ASR::expr_t *empty_space = ASRUtils::EXPR(ASR::make_StringConstant_t(
             al, loc, s2c(al, ""), str_type_len));
         ASR::stmt_t* empty_file_write_endl = ASRUtils::STMT(ASR::make_FileWrite_t(al, loc,
-                0, unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr));
+                0, unit, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr, true, nullptr));
         for( int i = n_dims - 1; i >= 0; i-- ) {
             ASR::do_loop_head_t head;
             head.m_v = idx_vars[i];
@@ -223,11 +257,11 @@ public:
                     format_args.push_back(al, string_format);
                     write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
                         al, loc, i, unit, nullptr, nullptr, nullptr,
-                        format_args.p, format_args.size(), nullptr, empty_space, nullptr));
+                        format_args.p, format_args.size(), nullptr, empty_space, nullptr, true, nullptr));
                 } else {
                     write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
                         al, loc, i, unit, nullptr, nullptr, nullptr,
-                        print_args.p, print_args.size(), nullptr, nullptr, nullptr));
+                        print_args.p, print_args.size(), nullptr, nullptr, nullptr, true, nullptr));
                 }
                 doloop_body.push_back(al, write_stmt);
             } else {
@@ -244,7 +278,7 @@ public:
         body.from_pointer_n_copy(al, write_body.data(), write_body.size());
         ASR::stmt_t* write_stmt = ASRUtils::STMT(ASR::make_FileWrite_t(
             al, x.base.base.loc, x.m_label, x.m_unit, x.m_iomsg,
-            x.m_iostat, x.m_id, body.p, body.size(), x.m_separator, x.m_end, x.m_overloaded));
+            x.m_iostat, x.m_id, body.p, body.size(), x.m_separator, x.m_end, x.m_overloaded, true, nullptr));
         pass_result.push_back(al, write_stmt);
         write_body.clear();
     }
@@ -321,4 +355,4 @@ void pass_replace_print_arr(Allocator &al, ASR::TranslationUnit_t &unit,
 }
 
 
-} // namespace LCompilers
\ No newline at end of file
+} // namespace LCompilers
