@@ -6,10 +6,10 @@
 #include <libasr/pass/unique_symbols.h>
 #include <libasr/pass/pass_utils.h>
 #include <unordered_map>
-#include <set>
+#include<unordered_set>
 
 
-extern std::string lcompilers_unique_ID;
+extern std::string lcompilers_unique_ID_separate_compilation;
 
 /*
 ASR pass for replacing symbol names with some new name, mostly because
@@ -40,38 +40,58 @@ uint64_t static inline get_hash(ASR::asr_t *node)
 class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
     public:
     std::unordered_map<ASR::symbol_t*, std::string> sym_to_renamed;
+    bool intrinsic_module_name_mangling;
     bool module_name_mangling;
     bool global_symbols_mangling;
     bool intrinsic_symbols_mangling;
     bool all_symbols_mangling;
     bool bindc_mangling = false;
     bool fortran_mangling;
+    bool c_mangling;
+    bool mangle_underscore_external;
     bool should_mangle = false;
     std::vector<std::string> parent_function_name;
     std::string module_name = "";
     SymbolTable* current_scope = nullptr;
 
-    SymbolRenameVisitor(bool mm, bool gm, bool im, bool am, bool bcm, bool fm) :
-    module_name_mangling(mm), global_symbols_mangling(gm), intrinsic_symbols_mangling(im),
-    all_symbols_mangling(am), bindc_mangling(bcm), fortran_mangling(fm) {}
+    SymbolRenameVisitor(bool imm, bool mm, bool gm, bool im, bool am, bool bcm, bool fm, bool cm, bool mue) :
+    intrinsic_module_name_mangling(imm), module_name_mangling(mm), global_symbols_mangling(gm), intrinsic_symbols_mangling(im),
+    all_symbols_mangling(am), bindc_mangling(bcm), fortran_mangling(fm), c_mangling(cm), mangle_underscore_external(mue) {}
 
 
+    const std::unordered_set<std::string> reserved_keywords_c = {
+         "_Alignas", "_Alignof", "_Atomic", "_Bool", "_Complex", "_Generic",
+         "_Imaginary", "_Noreturn", "_Static_assert", "_Thread_local", "auto",
+         "break", "case", "char", "_Bool", "const", "continue", "default", "do",
+         "double", "else", "enum", "extern", "float", "for", "goto", "if", "int",
+         "long", "register", "return", "short", "signed", "sizeof", "static",
+         "struct", "switch", "typedef", "union", "unsigned", "void", "volatile", "while"
+    };
+
+    //TODO: Implement other backends mangling when refactoring the pass infrastructure
+    void mangle_c(ASR::symbol_t* sym, const std::string& name){
+        if (reserved_keywords_c.find(name) != reserved_keywords_c.end()) {
+            sym_to_renamed[sym] = "_xx_"+std::string(name)+"_xx_";
+        }
+        return;
+    }
+
     std::string update_name(std::string curr_name) {
         if (startswith(curr_name, "_lpython") || startswith(curr_name, "_lfortran") ) {
             return curr_name;
         } else if (startswith(curr_name, "_lcompilers_") && current_scope) {
             // mangle intrinsic functions
             uint64_t hash = get_hash(current_scope->asr_owner);
-            return module_name + curr_name + "_" + std::to_string(hash) + "_" + lcompilers_unique_ID;
+            return module_name + curr_name + "_" + std::to_string(hash) + "_" + lcompilers_unique_ID_separate_compilation;
         } else if (parent_function_name.size() > 0) {
             // add parent function name to suffix
             std::string name = module_name + curr_name + "_";
             for (auto &a: parent_function_name) {
                 name += a + "_";
             }
-            return name + lcompilers_unique_ID;
+            return name + lcompilers_unique_ID_separate_compilation;
         }
-        return module_name + curr_name + "_" + lcompilers_unique_ID;
+        return module_name + curr_name + "_" + lcompilers_unique_ID_separate_compilation;
     }
 
     void visit_TranslationUnit(const ASR::TranslationUnit_t &x) {
@@ -101,11 +121,20 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
         module_name = std::string(x.m_name) + "_";
         if (all_symbols_mangling || module_name_mangling || should_mangle) {
             sym_to_renamed[sym] = update_name(x.m_name);
+        } else if ( intrinsic_module_name_mangling && x.m_intrinsic ) {
+            sym_to_renamed[sym] = update_name(x.m_name);
         }
         if ((global_symbols_mangling && startswith(x.m_name, "_global_symbols"))) {
             should_mangle = true;
         }
         for (auto &a : x.m_symtab->get_scope()) {
+            if ( intrinsic_module_name_mangling && startswith(x.m_name, "lfortran_intrinsic") ) {
+                // mangle functions / variables declared inside intrinsic modules
+                ASR::symbol_t *sym = a.second;
+                if (sym_to_renamed.find(sym) == sym_to_renamed.end()) {
+                    sym_to_renamed[sym] = update_name(ASRUtils::symbol_name(sym));
+                }
+            }
             visit_symbol(*a.second);
         }
         should_mangle = should_mangle_copy;
@@ -147,11 +176,30 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
                         "f" + std::string(x.m_name));
                 }
             }
+            if ( c_mangling ) {
+                ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
+                mangle_c(sym , std::string(x.m_name));
+            }
         }
-        if (intrinsic_symbols_mangling && startswith(x.m_name, "_lcompilers_")) {
+        if (intrinsic_symbols_mangling && (startswith(x.m_name, "_lcompilers_") || startswith(x.m_name, "__lcompilers"))) {
             ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
             sym_to_renamed[sym] = update_name(x.m_name);
         }
+        if (mangle_underscore_external) {
+            bool is_external_interface = (f_type->m_deftype == ASR::deftypeType::Interface &&
+                                          f_type->m_abi != ASR::abiType::Intrinsic &&
+                                          !f_type->m_module);
+            if (is_external_interface) {
+                std::string name = x.m_name;
+                bool has_reserved_prefix = (startswith(name, "_lfortran") ||
+                                            startswith(name, "_lpython") ||
+                                            startswith(name, "_lcompilers"));
+                if (!has_reserved_prefix && !name.empty() && name.back() != '_') {
+                    ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
+                    sym_to_renamed[sym] = name + "_";
+                }
+            }
+        }
         for (auto &a : x.m_symtab->get_scope()) {
             bool nested_function = is_nested_function(a.second);
             if (nested_function) {
@@ -181,6 +229,9 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
                     std::string(x.m_name));
             }
         }
+        if ( c_mangling ) {
+            mangle_c(sym , std::string(x.m_name));
+        }
     }
 
     void visit_GenericProcedure(const ASR::GenericProcedure_t &x) {
@@ -207,34 +258,38 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
                 sym_to_renamed[sym] = update_name(x.m_name);
             }
         }
+        if ( c_mangling ) {
+            ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
+            mangle_c(sym , std::string(x.m_name));
+        }
         for (auto &a : x.m_symtab->get_scope()) {
             this->visit_symbol(*a.second);
         }
     }
 
-    void visit_StructType(const ASR::StructType_t &x) {
+    void visit_Struct(const ASR::Struct_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_EnumType(const ASR::EnumType_t &x) {
+    void visit_Enum(const ASR::Enum_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_UnionType(const ASR::UnionType_t &x) {
+    void visit_Union(const ASR::Union_t &x) {
         visit_symbols_2(x);
     }
 
-    void visit_ClassType(const ASR::ClassType_t &x) {
-        visit_symbols_2(x);
-    }
-
-    void visit_ClassProcedure(const ASR::ClassProcedure_t &x) {
+    void visit_StructMethodDeclaration(const ASR::StructMethodDeclaration_t &x) {
         if (bindc_mangling || x.m_abi != ASR::abiType::BindC) {
             if (all_symbols_mangling || should_mangle) {
                 ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
                 sym_to_renamed[sym] = update_name(x.m_name);
             }
         }
+        if (c_mangling ) {
+            ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
+            mangle_c(sym , std::string(x.m_name));
+        }
     }
 
     template <typename T>
@@ -243,6 +298,10 @@ class SymbolRenameVisitor: public ASR::BaseWalkVisitor<SymbolRenameVisitor> {
             ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
             sym_to_renamed[sym] = update_name(x.m_name);
         }
+        if ( c_mangling ) {
+            ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
+            mangle_c(sym , std::string(x.m_name));
+        }
         for (auto &a : x.m_symtab->get_scope()) {
             this->visit_symbol(*a.second);
         }
@@ -406,15 +465,15 @@ class UniqueSymbolVisitor: public ASR::BaseWalkVisitor<UniqueSymbolVisitor> {
         current_scope = current_scope_copy;
     }
 
-    void visit_StructType(const ASR::StructType_t &x) {
+    void visit_Struct(const ASR::Struct_t &x) {
         update_symbols_2(x);
     }
 
-    void visit_EnumType(const ASR::EnumType_t &x) {
+    void visit_Enum(const ASR::Enum_t &x) {
         update_symbols_2(x);
     }
 
-    void visit_UnionType(const ASR::UnionType_t &x) {
+    void visit_Union(const ASR::Union_t &x) {
         update_symbols_2(x);
     }
 
@@ -434,28 +493,8 @@ class UniqueSymbolVisitor: public ASR::BaseWalkVisitor<UniqueSymbolVisitor> {
         }
     }
 
-    void visit_ClassType(const ASR::ClassType_t &x) {
-        ASR::ClassType_t& xx = const_cast<ASR::ClassType_t&>(x);
-        ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
-        if (sym_to_new_name.find(sym) != sym_to_new_name.end()) {
-            xx.m_name = s2c(al, sym_to_new_name[sym]);
-        }
-        std::map<std::string, ASR::symbol_t*> current_scope_copy = current_scope;
-        current_scope = x.m_symtab->get_scope();
-        for (auto &a : x.m_symtab->get_scope()) {
-            visit_symbol(*a.second);
-        }
-        for (auto &a: current_scope) {
-            if (sym_to_new_name.find(a.second) != sym_to_new_name.end()) {
-                xx.m_symtab->erase_symbol(a.first);
-                xx.m_symtab->add_symbol(sym_to_new_name[a.second], a.second);
-            }
-        }
-        current_scope = current_scope_copy;
-    }
-
-    void visit_ClassProcedure(const ASR::ClassProcedure_t &x) {
-        ASR::ClassProcedure_t& xx = const_cast<ASR::ClassProcedure_t&>(x);
+    void visit_StructMethodDeclaration(const ASR::StructMethodDeclaration_t &x) {
+        ASR::StructMethodDeclaration_t& xx = const_cast<ASR::StructMethodDeclaration_t&>(x);
         ASR::symbol_t *sym = ASR::down_cast<ASR::symbol_t>((ASR::asr_t*)&x);
         if (sym_to_new_name.find(sym) != sym_to_new_name.end()) {
             xx.m_name = s2c(al, sym_to_new_name[sym]);
@@ -510,7 +549,7 @@ void pass_unique_symbols(Allocator &al, ASR::TranslationUnit_t &unit,
      * MANGLING_OPTION="--module-mangling"
      * MANGLING_OPTION="--global-mangling"
      * MANGLING_OPTION="--intrinsic-mangling"
-     * COMPILER_SPECIFIC_OPTION="--generate-object-code" // LFortran
+     * COMPILER_SPECIFIC_OPTION="--separate-compilation" // LFortran
      * COMPILER_SPECIFIC_OPTION="--separate-compilation" // LPython
      * Usage:
      *    `$MANGLING_OPTION $COMPILER_SPECIFIC_OPTION`
@@ -518,24 +557,30 @@ void pass_unique_symbols(Allocator &al, ASR::TranslationUnit_t &unit,
      *    `$MANGLING_OPTIONS --mangle-underscore [$COMPILER_SPECIFIC_OPTION]`
      *    * `--apply-fortran-mangling [$MANGLING_OPTION] [$COMPILER_SPECIFIC_OPTION]`
      */
-    bool any_present = (pass_options.module_name_mangling || pass_options.global_symbols_mangling ||
+    bool any_present = (pass_options.intrinsic_module_name_mangling || pass_options.module_name_mangling || pass_options.global_symbols_mangling ||
                     pass_options.intrinsic_symbols_mangling || pass_options.all_symbols_mangling ||
-                    pass_options.bindc_mangling || pass_options.fortran_mangling);
+                    pass_options.bindc_mangling || pass_options.fortran_mangling ||
+                    pass_options.mangle_underscore_external);
     if (pass_options.mangle_underscore) {
-        lcompilers_unique_ID = "";
+        lcompilers_unique_ID_separate_compilation = "";
     }
-    if (!any_present || (!(pass_options.mangle_underscore ||
-            pass_options.fortran_mangling) && lcompilers_unique_ID.empty())) {
-        // `--mangle-underscore` doesn't require `lcompilers_unique_ID`
-        // `lcompilers_unique_ID` is not mandatory for `--apply-fortran-mangling`
+    if ((!any_present || (!(pass_options.mangle_underscore ||
+            pass_options.fortran_mangling || pass_options.mangle_underscore_external) &&
+            lcompilers_unique_ID_separate_compilation.empty())) &&
+                !pass_options.c_mangling) {
+        // `--mangle-underscore` doesn't require `lcompilers_unique_ID_separate_compilation`
+        // `lcompilers_unique_ID_separate_compilation` is not mandatory for `--apply-fortran-mangling`
         return;
     }
-    SymbolRenameVisitor v(pass_options.module_name_mangling,
+    SymbolRenameVisitor v(pass_options.intrinsic_module_name_mangling,
+                pass_options.module_name_mangling,
                 pass_options.global_symbols_mangling,
                 pass_options.intrinsic_symbols_mangling,
                 pass_options.all_symbols_mangling,
                 pass_options.bindc_mangling,
-                pass_options.fortran_mangling);
+                pass_options.fortran_mangling,
+                pass_options.c_mangling,
+                pass_options.mangle_underscore_external);
     v.visit_TranslationUnit(unit);
     UniqueSymbolVisitor u(al, v.sym_to_renamed);
     u.visit_TranslationUnit(unit);
